case RISCVVector::BI__builtin_rvv_th_vget_v:
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (isa<StructType>(Ops[0]->getType())) // For tuple type
          // Extract value from index (operand 1) of vtuple (operand 0)
          return Builder.CreateExtractValue(
            Ops[0],
            {(unsigned)cast<ConstantInt>(Ops[1])->getZExtValue()});
        auto *VecTy = cast<ScalableVectorType>(ResultType);
        auto *OpVecTy = cast<ScalableVectorType>(Ops[0]->getType());
        // Mask to only valid indices.
        unsigned MaxIndex = OpVecTy->getMinNumElements() / VecTy->getMinNumElements();
        assert(isPowerOf2_32(MaxIndex));
        Ops[1] = Builder.CreateZExt(Ops[1], Builder.getInt64Ty());
        Ops[1] = Builder.CreateAnd(Ops[1], MaxIndex - 1);
        Ops[1] = Builder.CreateMul(Ops[1],
                                   ConstantInt::get(Ops[1]->getType(),
                                                    VecTy->getMinNumElements()));
        return Builder.CreateExtractVector(ResultType, Ops[0], Ops[1]);
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vset_v:
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (isa<StructType>(ResultType)) // For tuple type
          // Insert value (operand 2) into index (operand 1) of vtuple (operand 0)
          return Builder.CreateInsertValue(
            Ops[0], Ops[2],
            {(unsigned)cast<ConstantInt>(Ops[1])->getZExtValue()});
        auto *ResVecTy = cast<ScalableVectorType>(ResultType);
        auto *VecTy = cast<ScalableVectorType>(Ops[2]->getType());
        // Mask to only valid indices.
        unsigned MaxIndex = ResVecTy->getMinNumElements() / VecTy->getMinNumElements();
        assert(isPowerOf2_32(MaxIndex));
        Ops[1] = Builder.CreateZExt(Ops[1], Builder.getInt64Ty());
        Ops[1] = Builder.CreateAnd(Ops[1], MaxIndex - 1);
        Ops[1] = Builder.CreateMul(Ops[1],
                                   ConstantInt::get(Ops[1]->getType(),
                                                    VecTy->getMinNumElements()));
        return Builder.CreateInsertVector(ResultType, Ops[0], Ops[2], Ops[1]);
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vlmul_ext_v:
  PolicyAttrs = 3;
IsMasked = false;

        return Builder.CreateInsertVector(ResultType,
                                          llvm::PoisonValue::get(ResultType),
                                          Ops[0], ConstantInt::get(Int64Ty, 0));
      break;
case RISCVVector::BI__builtin_rvv_th_vlmul_trunc_v:
  PolicyAttrs = 3;
IsMasked = false;
 {
        return Builder.CreateExtractVector(ResultType, Ops[0],
                                           ConstantInt::get(Int64Ty, 0));
      } break;
case RISCVVector::BI__builtin_rvv_th_vreinterpret_v:
  PolicyAttrs = 3;
IsMasked = false;

        if (ResultType->isIntOrIntVectorTy(1) ||
            Ops[0]->getType()->isIntOrIntVectorTy(1)) {
          assert(isa<ScalableVectorType>(ResultType) &&
                 isa<ScalableVectorType>(Ops[0]->getType()));

          LLVMContext &Context = CGM.getLLVMContext();
          ScalableVectorType *Boolean64Ty =
              ScalableVectorType::get(llvm::Type::getInt1Ty(Context), 64);

          if (ResultType->isIntOrIntVectorTy(1)) {
            // Casting from m1 vector integer -> vector boolean
            // Ex: <vscale x 8 x i8>
            //     --(bitcast)--------> <vscale x 64 x i1>
            //     --(vector_extract)-> <vscale x  8 x i1>
            llvm::Value *BitCast = Builder.CreateBitCast(Ops[0], Boolean64Ty);
            return Builder.CreateExtractVector(ResultType, BitCast,
                                               ConstantInt::get(Int64Ty, 0));
          } else {
            // Casting from vector boolean -> m1 vector integer
            // Ex: <vscale x  1 x i1>
            //       --(vector_insert)-> <vscale x 64 x i1>
            //       --(bitcast)-------> <vscale x  8 x i8>
            llvm::Value *Boolean64Val =
              Builder.CreateInsertVector(Boolean64Ty,
                                         llvm::PoisonValue::get(Boolean64Ty),
                                         Ops[0],
                                         ConstantInt::get(Int64Ty, 0));
            return Builder.CreateBitCast(Boolean64Val, ResultType);
          }
        }
        // Just simple type cast
        return Builder.CreateBitCast(Ops[0], ResultType);
      break;
case RISCVVector::BI__builtin_rvv_th_vundefined:
  PolicyAttrs = 3;
IsMasked = false;

        return llvm::PoisonValue::get(ResultType);
      break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx_tu:
  ID = Intrinsic::riscv_th_vaadd;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx:
  ID = Intrinsic::riscv_th_vaadd;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx_tum:
  ID = Intrinsic::riscv_th_vaadd_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx_m:
  ID = Intrinsic::riscv_th_vaadd_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx_tumu:
  ID = Intrinsic::riscv_th_vaadd_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vaadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vaadd_vx_mu:
  ID = Intrinsic::riscv_th_vaadd_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vadc_vvm_tu:
case RISCVVector::BI__builtin_rvv_th_vadc_vxm_tu:
  ID = Intrinsic::riscv_th_vadc;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadc_vvm:
case RISCVVector::BI__builtin_rvv_th_vadc_vxm:
  ID = Intrinsic::riscv_th_vadc;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vadd_vx_tu:
  ID = Intrinsic::riscv_th_vadd;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv:
case RISCVVector::BI__builtin_rvv_th_vadd_vx:
  ID = Intrinsic::riscv_th_vadd;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vadd_vx_tum:
  ID = Intrinsic::riscv_th_vadd_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vadd_vx_m:
  ID = Intrinsic::riscv_th_vadd_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vadd_vx_tumu:
  ID = Intrinsic::riscv_th_vadd_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vadd_vx_mu:
  ID = Intrinsic::riscv_th_vadd_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vand_vx_tu:
  ID = Intrinsic::riscv_th_vand;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv:
case RISCVVector::BI__builtin_rvv_th_vand_vx:
  ID = Intrinsic::riscv_th_vand;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vand_vx_tum:
  ID = Intrinsic::riscv_th_vand_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv_m:
case RISCVVector::BI__builtin_rvv_th_vand_vx_m:
  ID = Intrinsic::riscv_th_vand_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vand_vx_tumu:
  ID = Intrinsic::riscv_th_vand_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vand_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vand_vx_mu:
  ID = Intrinsic::riscv_th_vand_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vasub_vx_tu:
  ID = Intrinsic::riscv_th_vasub;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv:
case RISCVVector::BI__builtin_rvv_th_vasub_vx:
  ID = Intrinsic::riscv_th_vasub;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vasub_vx_tum:
  ID = Intrinsic::riscv_th_vasub_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vasub_vx_m:
  ID = Intrinsic::riscv_th_vasub_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vasub_vx_tumu:
  ID = Intrinsic::riscv_th_vasub_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vasub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vasub_vx_mu:
  ID = Intrinsic::riscv_th_vasub_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vcompress_vm_tu:
  ID = Intrinsic::riscv_th_vcompress;
  PolicyAttrs = 2;
IsMasked = false;

      // insert poison passthru
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vcompress_vm:
  ID = Intrinsic::riscv_th_vcompress;
  PolicyAttrs = 3;
IsMasked = false;

      // insert poison passthru
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx_tu:
  ID = Intrinsic::riscv_th_vdiv;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx:
  ID = Intrinsic::riscv_th_vdiv;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx_tum:
  ID = Intrinsic::riscv_th_vdiv_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv_m:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx_m:
  ID = Intrinsic::riscv_th_vdiv_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx_tumu:
  ID = Intrinsic::riscv_th_vdiv_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdiv_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vdiv_vx_mu:
  ID = Intrinsic::riscv_th_vdiv_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx_tu:
  ID = Intrinsic::riscv_th_vdivu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx:
  ID = Intrinsic::riscv_th_vdivu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx_tum:
  ID = Intrinsic::riscv_th_vdivu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx_m:
  ID = Intrinsic::riscv_th_vdivu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx_tumu:
  ID = Intrinsic::riscv_th_vdivu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vdivu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vdivu_vx_mu:
  ID = Intrinsic::riscv_th_vdivu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf_tu:
  ID = Intrinsic::riscv_th_vfadd;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf:
  ID = Intrinsic::riscv_th_vfadd;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf_tum:
  ID = Intrinsic::riscv_th_vfadd_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf_m:
  ID = Intrinsic::riscv_th_vfadd_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf_tumu:
  ID = Intrinsic::riscv_th_vfadd_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfadd_vf_mu:
  ID = Intrinsic::riscv_th_vfadd_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v_tu:
  ID = Intrinsic::riscv_th_vfclass;
  PolicyAttrs = 2;
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v:
  ID = Intrinsic::riscv_th_vfclass;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v_tum:
  ID = Intrinsic::riscv_th_vfclass_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v_m:
  ID = Intrinsic::riscv_th_vfclass_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v_tumu:
  ID = Intrinsic::riscv_th_vfclass_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfclass_v_mu:
  ID = Intrinsic::riscv_th_vfclass_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v_tu:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v_tum:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v_m:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v_tumu:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_x_v_mu:
  ID = Intrinsic::riscv_th_vfcvt_f_x_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v_tu:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v_tum:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v_m:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v_tumu:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_f_xu_v_mu:
  ID = Intrinsic::riscv_th_vfcvt_f_xu_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v_tu:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v_tum:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v_m:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v_tumu:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_x_f_v_mu:
  ID = Intrinsic::riscv_th_vfcvt_x_f_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v_tu:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v_tum:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v_m:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v_tumu:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfcvt_xu_f_v_mu:
  ID = Intrinsic::riscv_th_vfcvt_xu_f_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf_tu:
  ID = Intrinsic::riscv_th_vfdiv;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf:
  ID = Intrinsic::riscv_th_vfdiv;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf_tum:
  ID = Intrinsic::riscv_th_vfdiv_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf_m:
  ID = Intrinsic::riscv_th_vfdiv_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf_tumu:
  ID = Intrinsic::riscv_th_vfdiv_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfdiv_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfdiv_vf_mu:
  ID = Intrinsic::riscv_th_vfdiv_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf_tu:
  ID = Intrinsic::riscv_th_vfmacc;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf:
  ID = Intrinsic::riscv_th_vfmacc;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf_tum:
  ID = Intrinsic::riscv_th_vfmacc_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf_m:
  ID = Intrinsic::riscv_th_vfmacc_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf_tumu:
  ID = Intrinsic::riscv_th_vfmacc_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmacc_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmacc_vf_mu:
  ID = Intrinsic::riscv_th_vfmacc_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf_tu:
  ID = Intrinsic::riscv_th_vfmadd;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf:
  ID = Intrinsic::riscv_th_vfmadd;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf_tum:
  ID = Intrinsic::riscv_th_vfmadd_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf_m:
  ID = Intrinsic::riscv_th_vfmadd_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf_tumu:
  ID = Intrinsic::riscv_th_vfmadd_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmadd_vf_mu:
  ID = Intrinsic::riscv_th_vfmadd_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf_tu:
  ID = Intrinsic::riscv_th_vfmax;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf:
  ID = Intrinsic::riscv_th_vfmax;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf_tum:
  ID = Intrinsic::riscv_th_vfmax_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf_m:
  ID = Intrinsic::riscv_th_vfmax_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf_tumu:
  ID = Intrinsic::riscv_th_vfmax_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmax_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmax_vf_mu:
  ID = Intrinsic::riscv_th_vfmax_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmerge_vfm_tu:
  ID = Intrinsic::riscv_th_vfmerge;
  PolicyAttrs = 2;
IsMasked = false;

      // insert poison passthru
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vfmerge_vfm:
  ID = Intrinsic::riscv_th_vfmerge;
  PolicyAttrs = 3;
IsMasked = false;

      // insert poison passthru
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf_tu:
  ID = Intrinsic::riscv_th_vfmin;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf:
  ID = Intrinsic::riscv_th_vfmin;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf_tum:
  ID = Intrinsic::riscv_th_vfmin_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf_m:
  ID = Intrinsic::riscv_th_vfmin_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf_tumu:
  ID = Intrinsic::riscv_th_vfmin_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmin_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmin_vf_mu:
  ID = Intrinsic::riscv_th_vfmin_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf_tu:
  ID = Intrinsic::riscv_th_vfmsac;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf:
  ID = Intrinsic::riscv_th_vfmsac;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf_tum:
  ID = Intrinsic::riscv_th_vfmsac_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf_m:
  ID = Intrinsic::riscv_th_vfmsac_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf_tumu:
  ID = Intrinsic::riscv_th_vfmsac_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsac_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmsac_vf_mu:
  ID = Intrinsic::riscv_th_vfmsac_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf_tu:
  ID = Intrinsic::riscv_th_vfmsub;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf:
  ID = Intrinsic::riscv_th_vfmsub;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf_tum:
  ID = Intrinsic::riscv_th_vfmsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf_m:
  ID = Intrinsic::riscv_th_vfmsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf_tumu:
  ID = Intrinsic::riscv_th_vfmsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmsub_vf_mu:
  ID = Intrinsic::riscv_th_vfmsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf_tu:
  ID = Intrinsic::riscv_th_vfmul;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf:
  ID = Intrinsic::riscv_th_vfmul;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf_tum:
  ID = Intrinsic::riscv_th_vfmul_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf_m:
  ID = Intrinsic::riscv_th_vfmul_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf_tumu:
  ID = Intrinsic::riscv_th_vfmul_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmul_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfmul_vf_mu:
  ID = Intrinsic::riscv_th_vfmul_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfmv_v_f_tu:
  ID = Intrinsic::riscv_th_vfmv_v_f;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfmv_v_f:
  ID = Intrinsic::riscv_th_vfmv_v_f;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w_tu:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w_tum:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w_m:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w_tumu:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_f_w_mu:
  ID = Intrinsic::riscv_th_vfncvt_f_f_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w_tu:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w_tum:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w_m:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w_tumu:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_x_w_mu:
  ID = Intrinsic::riscv_th_vfncvt_f_x_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w_tu:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w_tum:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w_m:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w_tumu:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_f_xu_w_mu:
  ID = Intrinsic::riscv_th_vfncvt_f_xu_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w_tu:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w_tum:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w_m:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w_tumu:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_x_f_w_mu:
  ID = Intrinsic::riscv_th_vfncvt_x_f_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w_tu:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w_tum:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w_m:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w_tumu:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfncvt_xu_f_w_mu:
  ID = Intrinsic::riscv_th_vfncvt_xu_f_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf_tu:
  ID = Intrinsic::riscv_th_vfnmacc;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf:
  ID = Intrinsic::riscv_th_vfnmacc;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf_tum:
  ID = Intrinsic::riscv_th_vfnmacc_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf_m:
  ID = Intrinsic::riscv_th_vfnmacc_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf_tumu:
  ID = Intrinsic::riscv_th_vfnmacc_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfnmacc_vf_mu:
  ID = Intrinsic::riscv_th_vfnmacc_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf_tu:
  ID = Intrinsic::riscv_th_vfnmadd;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf:
  ID = Intrinsic::riscv_th_vfnmadd;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf_tum:
  ID = Intrinsic::riscv_th_vfnmadd_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf_m:
  ID = Intrinsic::riscv_th_vfnmadd_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf_tumu:
  ID = Intrinsic::riscv_th_vfnmadd_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfnmadd_vf_mu:
  ID = Intrinsic::riscv_th_vfnmadd_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf_tu:
  ID = Intrinsic::riscv_th_vfnmsac;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf:
  ID = Intrinsic::riscv_th_vfnmsac;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf_tum:
  ID = Intrinsic::riscv_th_vfnmsac_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf_m:
  ID = Intrinsic::riscv_th_vfnmsac_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf_tumu:
  ID = Intrinsic::riscv_th_vfnmsac_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfnmsac_vf_mu:
  ID = Intrinsic::riscv_th_vfnmsac_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf_tu:
  ID = Intrinsic::riscv_th_vfnmsub;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf:
  ID = Intrinsic::riscv_th_vfnmsub;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf_tum:
  ID = Intrinsic::riscv_th_vfnmsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf_m:
  ID = Intrinsic::riscv_th_vfnmsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf_tumu:
  ID = Intrinsic::riscv_th_vfnmsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfnmsub_vf_mu:
  ID = Intrinsic::riscv_th_vfnmsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf_tu:
  ID = Intrinsic::riscv_th_vfrdiv;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf:
  ID = Intrinsic::riscv_th_vfrdiv;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf_tum:
  ID = Intrinsic::riscv_th_vfrdiv_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf_m:
  ID = Intrinsic::riscv_th_vfrdiv_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf_tumu:
  ID = Intrinsic::riscv_th_vfrdiv_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrdiv_vf_mu:
  ID = Intrinsic::riscv_th_vfrdiv_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmax_vs_tu:
  ID = Intrinsic::riscv_th_vfredmax;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmax_vs:
  ID = Intrinsic::riscv_th_vfredmax;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmax_vs_tum:
  ID = Intrinsic::riscv_th_vfredmax_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmax_vs_m:
  ID = Intrinsic::riscv_th_vfredmax_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmin_vs_tu:
  ID = Intrinsic::riscv_th_vfredmin;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmin_vs:
  ID = Intrinsic::riscv_th_vfredmin;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmin_vs_tum:
  ID = Intrinsic::riscv_th_vfredmin_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredmin_vs_m:
  ID = Intrinsic::riscv_th_vfredmin_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfredosum_vs_tu:
  ID = Intrinsic::riscv_th_vfredosum;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredosum_vs:
  ID = Intrinsic::riscv_th_vfredosum;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredosum_vs_tum:
  ID = Intrinsic::riscv_th_vfredosum_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredosum_vs_m:
  ID = Intrinsic::riscv_th_vfredosum_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredsum_vs_tu:
  ID = Intrinsic::riscv_th_vfredsum;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredsum_vs:
  ID = Intrinsic::riscv_th_vfredsum;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredsum_vs_tum:
  ID = Intrinsic::riscv_th_vfredsum_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfredsum_vs_m:
  ID = Intrinsic::riscv_th_vfredsum_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf_tu:
  ID = Intrinsic::riscv_th_vfrsub;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf:
  ID = Intrinsic::riscv_th_vfrsub;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf_tum:
  ID = Intrinsic::riscv_th_vfrsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf_m:
  ID = Intrinsic::riscv_th_vfrsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf_tumu:
  ID = Intrinsic::riscv_th_vfrsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfrsub_vf_mu:
  ID = Intrinsic::riscv_th_vfrsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf_tu:
  ID = Intrinsic::riscv_th_vfsgnj;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf:
  ID = Intrinsic::riscv_th_vfsgnj;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf_tum:
  ID = Intrinsic::riscv_th_vfsgnj_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf_m:
  ID = Intrinsic::riscv_th_vfsgnj_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf_tumu:
  ID = Intrinsic::riscv_th_vfsgnj_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfsgnj_vf_mu:
  ID = Intrinsic::riscv_th_vfsgnj_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v_tu:
  ID = Intrinsic::riscv_th_vfsgnjn;
  PolicyAttrs = 2;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf_tu:
  ID = Intrinsic::riscv_th_vfsgnjn;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v:
  ID = Intrinsic::riscv_th_vfsgnjn;
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf:
  ID = Intrinsic::riscv_th_vfsgnjn;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v_tum:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 2;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf_tum:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v_m:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 3;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf_m:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v_tumu:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 0;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf_tumu:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfneg_v_mu:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 1;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjn_vf_mu:
  ID = Intrinsic::riscv_th_vfsgnjn_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf_tu:
  ID = Intrinsic::riscv_th_vfsgnjx;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v_tu:
  ID = Intrinsic::riscv_th_vfsgnjx;
  PolicyAttrs = 2;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf:
  ID = Intrinsic::riscv_th_vfsgnjx;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v:
  ID = Intrinsic::riscv_th_vfsgnjx;
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf_tum:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v_tum:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 2;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf_m:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v_m:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 3;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf_tumu:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v_tumu:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 0;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfsgnjx_vf_mu:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfabs_v_mu:
  ID = Intrinsic::riscv_th_vfsgnjx_mask;
  PolicyAttrs = 1;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          Ops.insert(Ops.begin() + 2, Ops[1]);
          // TODO: no policy in LLVM side for masked intrinsics.
          // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl
          IntrinsicTypes = {ResultType,
                            Ops[2]->getType(),
                            Ops.back()->getType()};
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
          // op1, po2, vl
          IntrinsicTypes = {ResultType,
                            Ops[1]->getType(), Ops[2]->getType()};
          Ops.insert(Ops.begin() + 2, Ops[1]);
          break;
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v_tu:
  ID = Intrinsic::riscv_th_vfsqrt;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v:
  ID = Intrinsic::riscv_th_vfsqrt;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v_tum:
  ID = Intrinsic::riscv_th_vfsqrt_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v_m:
  ID = Intrinsic::riscv_th_vfsqrt_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v_tumu:
  ID = Intrinsic::riscv_th_vfsqrt_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsqrt_v_mu:
  ID = Intrinsic::riscv_th_vfsqrt_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, round_mode, vl)
      // Masked:   (passthru, op0, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf_tu:
  ID = Intrinsic::riscv_th_vfsub;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf:
  ID = Intrinsic::riscv_th_vfsub;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf_tum:
  ID = Intrinsic::riscv_th_vfsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf_m:
  ID = Intrinsic::riscv_th_vfsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf_tumu:
  ID = Intrinsic::riscv_th_vfsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfsub_vf_mu:
  ID = Intrinsic::riscv_th_vfsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf_tu:
  ID = Intrinsic::riscv_th_vfwadd;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf:
  ID = Intrinsic::riscv_th_vfwadd;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf_tum:
  ID = Intrinsic::riscv_th_vfwadd_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf_m:
  ID = Intrinsic::riscv_th_vfwadd_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf_tumu:
  ID = Intrinsic::riscv_th_vfwadd_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_vf_mu:
  ID = Intrinsic::riscv_th_vfwadd_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf_tu:
  ID = Intrinsic::riscv_th_vfwadd_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf:
  ID = Intrinsic::riscv_th_vfwadd_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf_tum:
  ID = Intrinsic::riscv_th_vfwadd_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv_m:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf_m:
  ID = Intrinsic::riscv_th_vfwadd_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf_tumu:
  ID = Intrinsic::riscv_th_vfwadd_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwadd_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwadd_wf_mu:
  ID = Intrinsic::riscv_th_vfwadd_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v_tu:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v_tum:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v_m:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v_tumu:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_f_v_mu:
  ID = Intrinsic::riscv_th_vfwcvt_f_f_v_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v_tu:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v_tum:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v_m:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v_tumu:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_x_v_mu:
  ID = Intrinsic::riscv_th_vfwcvt_f_x_v_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v_tu:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v_tum:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v_m:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v_tumu:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_f_xu_v_mu:
  ID = Intrinsic::riscv_th_vfwcvt_f_xu_v_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v_tu:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v_tum:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v_m:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v_tumu:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_x_f_v_mu:
  ID = Intrinsic::riscv_th_vfwcvt_x_f_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v_tu:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v_tum:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v_m:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v_tumu:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwcvt_xu_f_v_mu:
  ID = Intrinsic::riscv_th_vfwcvt_xu_f_v_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, frm, vl)
      // Masked:   (passthru, op0, mask, frm, vl, policy)
      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4) :
        (HasMaskedOff ? Ops.size() == 4 : Ops.size() == 3);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 1]); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 1]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 1]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf_tu:
  ID = Intrinsic::riscv_th_vfwmacc;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf:
  ID = Intrinsic::riscv_th_vfwmacc;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf_tum:
  ID = Intrinsic::riscv_th_vfwmacc_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf_m:
  ID = Intrinsic::riscv_th_vfwmacc_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf_tumu:
  ID = Intrinsic::riscv_th_vfwmacc_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwmacc_vf_mu:
  ID = Intrinsic::riscv_th_vfwmacc_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf_tu:
  ID = Intrinsic::riscv_th_vfwmsac;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf:
  ID = Intrinsic::riscv_th_vfwmsac;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf_tum:
  ID = Intrinsic::riscv_th_vfwmsac_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf_m:
  ID = Intrinsic::riscv_th_vfwmsac_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf_tumu:
  ID = Intrinsic::riscv_th_vfwmsac_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwmsac_vf_mu:
  ID = Intrinsic::riscv_th_vfwmsac_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf_tu:
  ID = Intrinsic::riscv_th_vfwmul;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf:
  ID = Intrinsic::riscv_th_vfwmul;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf_tum:
  ID = Intrinsic::riscv_th_vfwmul_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf_m:
  ID = Intrinsic::riscv_th_vfwmul_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf_tumu:
  ID = Intrinsic::riscv_th_vfwmul_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwmul_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwmul_vf_mu:
  ID = Intrinsic::riscv_th_vfwmul_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf_tu:
  ID = Intrinsic::riscv_th_vfwnmacc;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf:
  ID = Intrinsic::riscv_th_vfwnmacc;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf_tum:
  ID = Intrinsic::riscv_th_vfwnmacc_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf_m:
  ID = Intrinsic::riscv_th_vfwnmacc_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf_tumu:
  ID = Intrinsic::riscv_th_vfwnmacc_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwnmacc_vf_mu:
  ID = Intrinsic::riscv_th_vfwnmacc_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf_tu:
  ID = Intrinsic::riscv_th_vfwnmsac;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf:
  ID = Intrinsic::riscv_th_vfwnmsac;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf_tum:
  ID = Intrinsic::riscv_th_vfwnmsac_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf_m:
  ID = Intrinsic::riscv_th_vfwnmsac_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf_tumu:
  ID = Intrinsic::riscv_th_vfwnmsac_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwnmsac_vf_mu:
  ID = Intrinsic::riscv_th_vfwnmsac_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasRoundModeOp = IsMasked ? Ops.size() == 6 : Ops.size() == 5;

      unsigned Offset = IsMasked ? 2 : 1;

      Operands.push_back(Ops[IsMasked ? 1 : 0]); // passthrough

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredosum_vs_tu:
  ID = Intrinsic::riscv_th_vfwredosum;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredosum_vs:
  ID = Intrinsic::riscv_th_vfwredosum;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredosum_vs_tum:
  ID = Intrinsic::riscv_th_vfwredosum_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredosum_vs_m:
  ID = Intrinsic::riscv_th_vfwredosum_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredsum_vs_tu:
  ID = Intrinsic::riscv_th_vfwredsum;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredsum_vs:
  ID = Intrinsic::riscv_th_vfwredsum;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredsum_vs_tum:
  ID = Intrinsic::riscv_th_vfwredsum_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwredsum_vs_m:
  ID = Intrinsic::riscv_th_vfwredsum_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf_tu:
  ID = Intrinsic::riscv_th_vfwsub;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf:
  ID = Intrinsic::riscv_th_vfwsub;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf_tum:
  ID = Intrinsic::riscv_th_vfwsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf_m:
  ID = Intrinsic::riscv_th_vfwsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf_tumu:
  ID = Intrinsic::riscv_th_vfwsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_vf_mu:
  ID = Intrinsic::riscv_th_vfwsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                        Ops.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf_tu:
  ID = Intrinsic::riscv_th_vfwsub_w;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf:
  ID = Intrinsic::riscv_th_vfwsub_w;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf_tum:
  ID = Intrinsic::riscv_th_vfwsub_w_mask;
  PolicyAttrs = 2;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv_m:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf_m:
  ID = Intrinsic::riscv_th_vfwsub_w_mask;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf_tumu:
  ID = Intrinsic::riscv_th_vfwsub_w_mask;
  PolicyAttrs = 0;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vfwsub_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vfwsub_wf_mu:
  ID = Intrinsic::riscv_th_vfwsub_w_mask;
  PolicyAttrs = 1;
IsMasked = true;

    {
      // LLVM intrinsic
      // Unmasked: (passthru, op0, op1, round_mode, vl)
      // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, frm, vl)

      SmallVector<llvm::Value*, 7> Operands;
      bool HasMaskedOff = !(
          (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
          (!IsMasked && PolicyAttrs & RVV_VTA));
      bool HasRoundModeOp = IsMasked ?
        (HasMaskedOff ? Ops.size() == 6 : Ops.size() == 5) :
        (HasMaskedOff ? Ops.size() == 5 : Ops.size() == 4);

      unsigned Offset = IsMasked ?
          (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

      if (!HasMaskedOff)
        Operands.push_back(llvm::PoisonValue::get(ResultType));
      else
        Operands.push_back(Ops[IsMasked ? 1 : 0]);

      Operands.push_back(Ops[Offset]); // op0
      Operands.push_back(Ops[Offset + 1]); // op1

      if (IsMasked)
        Operands.push_back(Ops[0]); // mask

      if (HasRoundModeOp) {
        Operands.push_back(Ops[Offset + 2]); // frm
        Operands.push_back(Ops[Offset + 3]); // vl
      } else {
        Operands.push_back(ConstantInt::get(Ops[Offset + 2]->getType(), 7)); // frm
        Operands.push_back(Ops[Offset + 2]); // vl
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // if (IsMasked)
      //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(),
                        Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
    }
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v_tu:
  ID = Intrinsic::riscv_th_vid;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v:
  ID = Intrinsic::riscv_th_vid;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v_tum:
  ID = Intrinsic::riscv_th_vid_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v_m:
  ID = Intrinsic::riscv_th_vid_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v_tumu:
  ID = Intrinsic::riscv_th_vid_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vid_v_mu:
  ID = Intrinsic::riscv_th_vid_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m_tu:
  ID = Intrinsic::riscv_th_viota;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m:
  ID = Intrinsic::riscv_th_viota;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m_tum:
  ID = Intrinsic::riscv_th_viota_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m_m:
  ID = Intrinsic::riscv_th_viota_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m_tumu:
  ID = Intrinsic::riscv_th_viota_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_viota_m_mu:
  ID = Intrinsic::riscv_th_viota_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v_tu:
  ID = Intrinsic::riscv_th_vlb;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v:
  ID = Intrinsic::riscv_th_vlb;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v_tum:
  ID = Intrinsic::riscv_th_vlb_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v_m:
  ID = Intrinsic::riscv_th_vlb_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v_tumu:
  ID = Intrinsic::riscv_th_vlb_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlb_v_mu:
  ID = Intrinsic::riscv_th_vlb_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v_tu:
  ID = Intrinsic::riscv_th_vlbu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v:
  ID = Intrinsic::riscv_th_vlbu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v_tum:
  ID = Intrinsic::riscv_th_vlbu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v_m:
  ID = Intrinsic::riscv_th_vlbu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v_tumu:
  ID = Intrinsic::riscv_th_vlbu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlbu_v_mu:
  ID = Intrinsic::riscv_th_vlbu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle64_v_tu:
  ID = Intrinsic::riscv_th_vle;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v:
case RISCVVector::BI__builtin_rvv_th_vle16_v:
case RISCVVector::BI__builtin_rvv_th_vle32_v:
case RISCVVector::BI__builtin_rvv_th_vle64_v:
  ID = Intrinsic::riscv_th_vle;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle64_v_tum:
  ID = Intrinsic::riscv_th_vle_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v_m:
case RISCVVector::BI__builtin_rvv_th_vle16_v_m:
case RISCVVector::BI__builtin_rvv_th_vle32_v_m:
case RISCVVector::BI__builtin_rvv_th_vle64_v_m:
  ID = Intrinsic::riscv_th_vle_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle64_v_tumu:
  ID = Intrinsic::riscv_th_vle_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle64_v_mu:
  ID = Intrinsic::riscv_th_vle_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v_tu:
  ID = Intrinsic::riscv_th_vleff;
  PolicyAttrs = 2;
IsMasked = false;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v:
  ID = Intrinsic::riscv_th_vleff;
  PolicyAttrs = 3;
IsMasked = false;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v_tum:
  ID = Intrinsic::riscv_th_vleff_mask;
  PolicyAttrs = 2;
IsMasked = true;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v_m:
  ID = Intrinsic::riscv_th_vleff_mask;
  PolicyAttrs = 3;
IsMasked = true;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v_tumu:
  ID = Intrinsic::riscv_th_vleff_mask;
  PolicyAttrs = 0;
IsMasked = true;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vle8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vle64ff_v_mu:
  ID = Intrinsic::riscv_th_vleff_mask;
  PolicyAttrs = 1;
IsMasked = true;

        {
          if (IsMasked) {
            // Move mask to right before vl.
            std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
            if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
            IntrinsicTypes = {ResultType, Ops[4]->getType()};
          } else {
            if (PolicyAttrs & RVV_VTA)
              Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
            IntrinsicTypes = {ResultType, Ops[3]->getType()};
          }
          Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
          Value *NewVL = Ops[2];
          Ops.erase(Ops.begin() + 2);
          llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
          llvm::Value *LoadValue = Builder.CreateCall(F, Ops, "");
          llvm::Value *V = Builder.CreateExtractValue(LoadValue, {0});
          // Store new_vl.
          clang::CharUnits Align;
          if (IsMasked)
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(E->getNumArgs()-2)->getType());
          else
            Align = CGM.getNaturalPointeeTypeAlignment(E->getArg(1)->getType());
          llvm::Value *Val = Builder.CreateExtractValue(LoadValue, {1});
          Builder.CreateStore(Val, Address(NewVL, Val->getType(), Align));
          return V;
        }
    break;
case RISCVVector::BI__builtin_rvv_th_vlh_v_tu:
  ID = Intrinsic::riscv_th_vlh;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlh_v:
  ID = Intrinsic::riscv_th_vlh;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlh_v_tum:
  ID = Intrinsic::riscv_th_vlh_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlh_v_m:
  ID = Intrinsic::riscv_th_vlh_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlh_v_tumu:
  ID = Intrinsic::riscv_th_vlh_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlh_v_mu:
  ID = Intrinsic::riscv_th_vlh_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v_tu:
  ID = Intrinsic::riscv_th_vlhu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v:
  ID = Intrinsic::riscv_th_vlhu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v_tum:
  ID = Intrinsic::riscv_th_vlhu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v_m:
  ID = Intrinsic::riscv_th_vlhu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v_tumu:
  ID = Intrinsic::riscv_th_vlhu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlhu_v_mu:
  ID = Intrinsic::riscv_th_vlhu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v_tu:
  ID = Intrinsic::riscv_th_vlsb;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v:
  ID = Intrinsic::riscv_th_vlsb;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v_tum:
  ID = Intrinsic::riscv_th_vlsb_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v_m:
  ID = Intrinsic::riscv_th_vlsb_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v_tumu:
  ID = Intrinsic::riscv_th_vlsb_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsb_v_mu:
  ID = Intrinsic::riscv_th_vlsb_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v_tu:
  ID = Intrinsic::riscv_th_vlsbu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v:
  ID = Intrinsic::riscv_th_vlsbu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v_tum:
  ID = Intrinsic::riscv_th_vlsbu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v_m:
  ID = Intrinsic::riscv_th_vlsbu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v_tumu:
  ID = Intrinsic::riscv_th_vlsbu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsbu_v_mu:
  ID = Intrinsic::riscv_th_vlsbu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlse16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlse32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlse64_v_tu:
  ID = Intrinsic::riscv_th_vlse;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v:
case RISCVVector::BI__builtin_rvv_th_vlse16_v:
case RISCVVector::BI__builtin_rvv_th_vlse32_v:
case RISCVVector::BI__builtin_rvv_th_vlse64_v:
  ID = Intrinsic::riscv_th_vlse;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlse16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlse32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlse64_v_tum:
  ID = Intrinsic::riscv_th_vlse_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlse16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlse32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlse64_v_m:
  ID = Intrinsic::riscv_th_vlse_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlse16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlse32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlse64_v_tumu:
  ID = Intrinsic::riscv_th_vlse_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlse8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlse16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlse32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlse64_v_mu:
  ID = Intrinsic::riscv_th_vlse_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v_tu:
  ID = Intrinsic::riscv_th_vlseg2b;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v:
  ID = Intrinsic::riscv_th_vlseg2b;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v_tum:
  ID = Intrinsic::riscv_th_vlseg2b_mask;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v_m:
  ID = Intrinsic::riscv_th_vlseg2b_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg2b_mask;
  NF = 2;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2b_v_mu:
  ID = Intrinsic::riscv_th_vlseg2b_mask;
  NF = 2;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg2e;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v:
  ID = Intrinsic::riscv_th_vlseg2e;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg2e_mask;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v_m:
  ID = Intrinsic::riscv_th_vlseg2e_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg2e_mask;
  NF = 2;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg2e_mask;
  NF = 2;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg2eff;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v:
  ID = Intrinsic::riscv_th_vlseg2eff;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg2eff_mask;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg2eff_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg2eff_mask;
  NF = 2;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg2e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg2eff_mask;
  NF = 2;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v_tu:
  ID = Intrinsic::riscv_th_vlseg2h;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v:
  ID = Intrinsic::riscv_th_vlseg2h;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v_tum:
  ID = Intrinsic::riscv_th_vlseg2h_mask;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v_m:
  ID = Intrinsic::riscv_th_vlseg2h_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg2h_mask;
  NF = 2;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2h_v_mu:
  ID = Intrinsic::riscv_th_vlseg2h_mask;
  NF = 2;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v_tu:
  ID = Intrinsic::riscv_th_vlseg2w;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v:
  ID = Intrinsic::riscv_th_vlseg2w;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v_tum:
  ID = Intrinsic::riscv_th_vlseg2w_mask;
  NF = 2;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v_m:
  ID = Intrinsic::riscv_th_vlseg2w_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg2w_mask;
  NF = 2;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg2w_v_mu:
  ID = Intrinsic::riscv_th_vlseg2w_mask;
  NF = 2;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v_tu:
  ID = Intrinsic::riscv_th_vlseg3b;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v:
  ID = Intrinsic::riscv_th_vlseg3b;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v_tum:
  ID = Intrinsic::riscv_th_vlseg3b_mask;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v_m:
  ID = Intrinsic::riscv_th_vlseg3b_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg3b_mask;
  NF = 3;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3b_v_mu:
  ID = Intrinsic::riscv_th_vlseg3b_mask;
  NF = 3;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg3e;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v:
  ID = Intrinsic::riscv_th_vlseg3e;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg3e_mask;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v_m:
  ID = Intrinsic::riscv_th_vlseg3e_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg3e_mask;
  NF = 3;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg3e_mask;
  NF = 3;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg3eff;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v:
  ID = Intrinsic::riscv_th_vlseg3eff;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg3eff_mask;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg3eff_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg3eff_mask;
  NF = 3;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg3e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg3eff_mask;
  NF = 3;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v_tu:
  ID = Intrinsic::riscv_th_vlseg3h;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v:
  ID = Intrinsic::riscv_th_vlseg3h;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v_tum:
  ID = Intrinsic::riscv_th_vlseg3h_mask;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v_m:
  ID = Intrinsic::riscv_th_vlseg3h_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg3h_mask;
  NF = 3;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3h_v_mu:
  ID = Intrinsic::riscv_th_vlseg3h_mask;
  NF = 3;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v_tu:
  ID = Intrinsic::riscv_th_vlseg3w;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v:
  ID = Intrinsic::riscv_th_vlseg3w;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v_tum:
  ID = Intrinsic::riscv_th_vlseg3w_mask;
  NF = 3;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v_m:
  ID = Intrinsic::riscv_th_vlseg3w_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg3w_mask;
  NF = 3;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg3w_v_mu:
  ID = Intrinsic::riscv_th_vlseg3w_mask;
  NF = 3;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v_tu:
  ID = Intrinsic::riscv_th_vlseg4b;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v:
  ID = Intrinsic::riscv_th_vlseg4b;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v_tum:
  ID = Intrinsic::riscv_th_vlseg4b_mask;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v_m:
  ID = Intrinsic::riscv_th_vlseg4b_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg4b_mask;
  NF = 4;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4b_v_mu:
  ID = Intrinsic::riscv_th_vlseg4b_mask;
  NF = 4;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg4e;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v:
  ID = Intrinsic::riscv_th_vlseg4e;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg4e_mask;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v_m:
  ID = Intrinsic::riscv_th_vlseg4e_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg4e_mask;
  NF = 4;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg4e_mask;
  NF = 4;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg4eff;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v:
  ID = Intrinsic::riscv_th_vlseg4eff;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg4eff_mask;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg4eff_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg4eff_mask;
  NF = 4;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg4e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg4eff_mask;
  NF = 4;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v_tu:
  ID = Intrinsic::riscv_th_vlseg4h;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v:
  ID = Intrinsic::riscv_th_vlseg4h;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v_tum:
  ID = Intrinsic::riscv_th_vlseg4h_mask;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v_m:
  ID = Intrinsic::riscv_th_vlseg4h_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg4h_mask;
  NF = 4;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4h_v_mu:
  ID = Intrinsic::riscv_th_vlseg4h_mask;
  NF = 4;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v_tu:
  ID = Intrinsic::riscv_th_vlseg4w;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v:
  ID = Intrinsic::riscv_th_vlseg4w;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v_tum:
  ID = Intrinsic::riscv_th_vlseg4w_mask;
  NF = 4;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v_m:
  ID = Intrinsic::riscv_th_vlseg4w_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg4w_mask;
  NF = 4;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg4w_v_mu:
  ID = Intrinsic::riscv_th_vlseg4w_mask;
  NF = 4;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v_tu:
  ID = Intrinsic::riscv_th_vlseg5b;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v:
  ID = Intrinsic::riscv_th_vlseg5b;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v_tum:
  ID = Intrinsic::riscv_th_vlseg5b_mask;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v_m:
  ID = Intrinsic::riscv_th_vlseg5b_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg5b_mask;
  NF = 5;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5b_v_mu:
  ID = Intrinsic::riscv_th_vlseg5b_mask;
  NF = 5;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg5e;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v:
  ID = Intrinsic::riscv_th_vlseg5e;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg5e_mask;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v_m:
  ID = Intrinsic::riscv_th_vlseg5e_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg5e_mask;
  NF = 5;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg5e_mask;
  NF = 5;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg5eff;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v:
  ID = Intrinsic::riscv_th_vlseg5eff;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg5eff_mask;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg5eff_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg5eff_mask;
  NF = 5;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg5e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg5eff_mask;
  NF = 5;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v_tu:
  ID = Intrinsic::riscv_th_vlseg5h;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v:
  ID = Intrinsic::riscv_th_vlseg5h;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v_tum:
  ID = Intrinsic::riscv_th_vlseg5h_mask;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v_m:
  ID = Intrinsic::riscv_th_vlseg5h_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg5h_mask;
  NF = 5;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5h_v_mu:
  ID = Intrinsic::riscv_th_vlseg5h_mask;
  NF = 5;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v_tu:
  ID = Intrinsic::riscv_th_vlseg5w;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v:
  ID = Intrinsic::riscv_th_vlseg5w;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v_tum:
  ID = Intrinsic::riscv_th_vlseg5w_mask;
  NF = 5;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v_m:
  ID = Intrinsic::riscv_th_vlseg5w_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg5w_mask;
  NF = 5;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg5w_v_mu:
  ID = Intrinsic::riscv_th_vlseg5w_mask;
  NF = 5;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v_tu:
  ID = Intrinsic::riscv_th_vlseg6b;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v:
  ID = Intrinsic::riscv_th_vlseg6b;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v_tum:
  ID = Intrinsic::riscv_th_vlseg6b_mask;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v_m:
  ID = Intrinsic::riscv_th_vlseg6b_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg6b_mask;
  NF = 6;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6b_v_mu:
  ID = Intrinsic::riscv_th_vlseg6b_mask;
  NF = 6;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg6e;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v:
  ID = Intrinsic::riscv_th_vlseg6e;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg6e_mask;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v_m:
  ID = Intrinsic::riscv_th_vlseg6e_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg6e_mask;
  NF = 6;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg6e_mask;
  NF = 6;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg6eff;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v:
  ID = Intrinsic::riscv_th_vlseg6eff;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg6eff_mask;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg6eff_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg6eff_mask;
  NF = 6;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg6e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg6eff_mask;
  NF = 6;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v_tu:
  ID = Intrinsic::riscv_th_vlseg6h;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v:
  ID = Intrinsic::riscv_th_vlseg6h;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v_tum:
  ID = Intrinsic::riscv_th_vlseg6h_mask;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v_m:
  ID = Intrinsic::riscv_th_vlseg6h_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg6h_mask;
  NF = 6;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6h_v_mu:
  ID = Intrinsic::riscv_th_vlseg6h_mask;
  NF = 6;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v_tu:
  ID = Intrinsic::riscv_th_vlseg6w;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v:
  ID = Intrinsic::riscv_th_vlseg6w;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v_tum:
  ID = Intrinsic::riscv_th_vlseg6w_mask;
  NF = 6;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v_m:
  ID = Intrinsic::riscv_th_vlseg6w_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg6w_mask;
  NF = 6;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg6w_v_mu:
  ID = Intrinsic::riscv_th_vlseg6w_mask;
  NF = 6;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v_tu:
  ID = Intrinsic::riscv_th_vlseg7b;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v:
  ID = Intrinsic::riscv_th_vlseg7b;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v_tum:
  ID = Intrinsic::riscv_th_vlseg7b_mask;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v_m:
  ID = Intrinsic::riscv_th_vlseg7b_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg7b_mask;
  NF = 7;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7b_v_mu:
  ID = Intrinsic::riscv_th_vlseg7b_mask;
  NF = 7;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg7e;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v:
  ID = Intrinsic::riscv_th_vlseg7e;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg7e_mask;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v_m:
  ID = Intrinsic::riscv_th_vlseg7e_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg7e_mask;
  NF = 7;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg7e_mask;
  NF = 7;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg7eff;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v:
  ID = Intrinsic::riscv_th_vlseg7eff;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg7eff_mask;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg7eff_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg7eff_mask;
  NF = 7;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg7e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg7eff_mask;
  NF = 7;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v_tu:
  ID = Intrinsic::riscv_th_vlseg7h;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v:
  ID = Intrinsic::riscv_th_vlseg7h;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v_tum:
  ID = Intrinsic::riscv_th_vlseg7h_mask;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v_m:
  ID = Intrinsic::riscv_th_vlseg7h_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg7h_mask;
  NF = 7;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7h_v_mu:
  ID = Intrinsic::riscv_th_vlseg7h_mask;
  NF = 7;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v_tu:
  ID = Intrinsic::riscv_th_vlseg7w;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v:
  ID = Intrinsic::riscv_th_vlseg7w;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v_tum:
  ID = Intrinsic::riscv_th_vlseg7w_mask;
  NF = 7;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v_m:
  ID = Intrinsic::riscv_th_vlseg7w_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg7w_mask;
  NF = 7;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg7w_v_mu:
  ID = Intrinsic::riscv_th_vlseg7w_mask;
  NF = 7;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v_tu:
  ID = Intrinsic::riscv_th_vlseg8b;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v:
  ID = Intrinsic::riscv_th_vlseg8b;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v_tum:
  ID = Intrinsic::riscv_th_vlseg8b_mask;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v_m:
  ID = Intrinsic::riscv_th_vlseg8b_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v_tumu:
  ID = Intrinsic::riscv_th_vlseg8b_mask;
  NF = 8;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8b_v_mu:
  ID = Intrinsic::riscv_th_vlseg8b_mask;
  NF = 8;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v_tu:
  ID = Intrinsic::riscv_th_vlseg8e;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v:
  ID = Intrinsic::riscv_th_vlseg8e;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v_tum:
  ID = Intrinsic::riscv_th_vlseg8e_mask;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v_m:
  ID = Intrinsic::riscv_th_vlseg8e_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v_tumu:
  ID = Intrinsic::riscv_th_vlseg8e_mask;
  NF = 8;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64_v_mu:
  ID = Intrinsic::riscv_th_vlseg8e_mask;
  NF = 8;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v_tu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v_tu:
  ID = Intrinsic::riscv_th_vlseg8eff;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v:
  ID = Intrinsic::riscv_th_vlseg8eff;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v_tum:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v_tum:
  ID = Intrinsic::riscv_th_vlseg8eff_mask;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v_m:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v_m:
  ID = Intrinsic::riscv_th_vlseg8eff_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v_tumu:
  ID = Intrinsic::riscv_th_vlseg8eff_mask;
  NF = 8;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8e8ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e16ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e32ff_v_mu:
case RISCVVector::BI__builtin_rvv_th_vlseg8e64ff_v_mu:
  ID = Intrinsic::riscv_th_vlseg8eff_mask;
  NF = 8;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // vl
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      // Get alignment from the new vl operand
      clang::CharUnits Align =
          CGM.getNaturalPointeeTypeAlignment(E->getArg(Offset + 1)->getType());

      llvm::Value *ReturnTuple = llvm::PoisonValue::get(ResultType);
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(LoadValue, {I});
        ReturnTuple = Builder.CreateInsertValue(ReturnTuple, V, {I});
      }

      // Store new_vl
      llvm::Value *V = Builder.CreateExtractValue(LoadValue, {NF});
      Builder.CreateStore(V, Address(Ops[Offset + 1], V->getType(), Align));

      if (ReturnValue.isNull())
        return ReturnTuple;
      else
        return Builder.CreateStore(ReturnTuple, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v_tu:
  ID = Intrinsic::riscv_th_vlseg8h;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v:
  ID = Intrinsic::riscv_th_vlseg8h;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v_tum:
  ID = Intrinsic::riscv_th_vlseg8h_mask;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v_m:
  ID = Intrinsic::riscv_th_vlseg8h_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v_tumu:
  ID = Intrinsic::riscv_th_vlseg8h_mask;
  NF = 8;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8h_v_mu:
  ID = Intrinsic::riscv_th_vlseg8h_mask;
  NF = 8;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v_tu:
  ID = Intrinsic::riscv_th_vlseg8w;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v:
  ID = Intrinsic::riscv_th_vlseg8w;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v_tum:
  ID = Intrinsic::riscv_th_vlseg8w_mask;
  NF = 8;
  PolicyAttrs = 2;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v_m:
  ID = Intrinsic::riscv_th_vlseg8w_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v_tumu:
  ID = Intrinsic::riscv_th_vlseg8w_mask;
  NF = 8;
  PolicyAttrs = 0;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlseg8w_v_mu:
  ID = Intrinsic::riscv_th_vlseg8w_mask;
  NF = 8;
  PolicyAttrs = 1;
IsMasked = true;

    {
      llvm::Type *ElementVectorType = cast<StructType>(ResultType)->elements()[0];
      IntrinsicTypes = {ElementVectorType, Ops.back()->getType()};
      SmallVector<llvm::Value*, 12> Operands;

      bool NoPassthru =
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) |
        (!IsMasked && (PolicyAttrs & RVV_VTA));
      unsigned Offset = IsMasked ? NoPassthru ? 1 : 2 : NoPassthru ? 0 : 1;

      if (NoPassthru) { // Push poison into passthru
        Operands.append(NF, llvm::PoisonValue::get(ElementVectorType));
      } else { // Push intrinsics operands into passthru
        llvm::Value *PassthruOperand = IsMasked ? Ops[1] : Ops[0];
        for (unsigned I = 0; I < NF; ++I)
          Operands.push_back(Builder.CreateExtractValue(PassthruOperand, {I}));
      }

      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 1]); // VL
      // TODO: no policy in LLVM side for masked intrinsics.
      if (IsMasked)
        Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);

      llvm::Value *LoadValue = Builder.CreateCall(F, Operands, "");
      if (ReturnValue.isNull())
        return LoadValue;
      else
        return Builder.CreateStore(LoadValue, ReturnValue.getValue());
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v_tu:
  ID = Intrinsic::riscv_th_vlsh;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v:
  ID = Intrinsic::riscv_th_vlsh;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v_tum:
  ID = Intrinsic::riscv_th_vlsh_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v_m:
  ID = Intrinsic::riscv_th_vlsh_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v_tumu:
  ID = Intrinsic::riscv_th_vlsh_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsh_v_mu:
  ID = Intrinsic::riscv_th_vlsh_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v_tu:
  ID = Intrinsic::riscv_th_vlshu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v:
  ID = Intrinsic::riscv_th_vlshu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v_tum:
  ID = Intrinsic::riscv_th_vlshu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v_m:
  ID = Intrinsic::riscv_th_vlshu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v_tumu:
  ID = Intrinsic::riscv_th_vlshu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlshu_v_mu:
  ID = Intrinsic::riscv_th_vlshu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v_tu:
  ID = Intrinsic::riscv_th_vlsw;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v:
  ID = Intrinsic::riscv_th_vlsw;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v_tum:
  ID = Intrinsic::riscv_th_vlsw_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v_m:
  ID = Intrinsic::riscv_th_vlsw_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v_tumu:
  ID = Intrinsic::riscv_th_vlsw_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlsw_v_mu:
  ID = Intrinsic::riscv_th_vlsw_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v_tu:
  ID = Intrinsic::riscv_th_vlswu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v:
  ID = Intrinsic::riscv_th_vlswu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v_tum:
  ID = Intrinsic::riscv_th_vlswu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v_m:
  ID = Intrinsic::riscv_th_vlswu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v_tumu:
  ID = Intrinsic::riscv_th_vlswu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlswu_v_mu:
  ID = Intrinsic::riscv_th_vlswu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v_tu:
  ID = Intrinsic::riscv_th_vlw;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v:
  ID = Intrinsic::riscv_th_vlw;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v_tum:
  ID = Intrinsic::riscv_th_vlw_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v_m:
  ID = Intrinsic::riscv_th_vlw_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v_tumu:
  ID = Intrinsic::riscv_th_vlw_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlw_v_mu:
  ID = Intrinsic::riscv_th_vlw_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v_tu:
  ID = Intrinsic::riscv_th_vlwu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v:
  ID = Intrinsic::riscv_th_vlwu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v_tum:
  ID = Intrinsic::riscv_th_vlwu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v_m:
  ID = Intrinsic::riscv_th_vlwu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v_tumu:
  ID = Intrinsic::riscv_th_vlwu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlwu_v_mu:
  ID = Intrinsic::riscv_th_vlwu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v_tu:
  ID = Intrinsic::riscv_th_vlxb;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v:
  ID = Intrinsic::riscv_th_vlxb;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v_tum:
  ID = Intrinsic::riscv_th_vlxb_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v_m:
  ID = Intrinsic::riscv_th_vlxb_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v_tumu:
  ID = Intrinsic::riscv_th_vlxb_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxb_v_mu:
  ID = Intrinsic::riscv_th_vlxb_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v_tu:
  ID = Intrinsic::riscv_th_vlxbu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v:
  ID = Intrinsic::riscv_th_vlxbu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v_tum:
  ID = Intrinsic::riscv_th_vlxbu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v_m:
  ID = Intrinsic::riscv_th_vlxbu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v_tumu:
  ID = Intrinsic::riscv_th_vlxbu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxbu_v_mu:
  ID = Intrinsic::riscv_th_vlxbu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v_tu:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v_tu:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v_tu:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v_tu:
  ID = Intrinsic::riscv_th_vlxe;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v:
  ID = Intrinsic::riscv_th_vlxe;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v_tum:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v_tum:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v_tum:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v_tum:
  ID = Intrinsic::riscv_th_vlxe_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v_m:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v_m:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v_m:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v_m:
  ID = Intrinsic::riscv_th_vlxe_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v_tumu:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v_tumu:
  ID = Intrinsic::riscv_th_vlxe_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vloxei8_v_mu:
case RISCVVector::BI__builtin_rvv_th_vloxei16_v_mu:
case RISCVVector::BI__builtin_rvv_th_vloxei32_v_mu:
case RISCVVector::BI__builtin_rvv_th_vloxei64_v_mu:
  ID = Intrinsic::riscv_th_vlxe_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v_tu:
  ID = Intrinsic::riscv_th_vlxh;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v:
  ID = Intrinsic::riscv_th_vlxh;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v_tum:
  ID = Intrinsic::riscv_th_vlxh_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v_m:
  ID = Intrinsic::riscv_th_vlxh_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v_tumu:
  ID = Intrinsic::riscv_th_vlxh_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxh_v_mu:
  ID = Intrinsic::riscv_th_vlxh_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v_tu:
  ID = Intrinsic::riscv_th_vlxhu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v:
  ID = Intrinsic::riscv_th_vlxhu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v_tum:
  ID = Intrinsic::riscv_th_vlxhu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v_m:
  ID = Intrinsic::riscv_th_vlxhu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v_tumu:
  ID = Intrinsic::riscv_th_vlxhu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxhu_v_mu:
  ID = Intrinsic::riscv_th_vlxhu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v_tu:
  ID = Intrinsic::riscv_th_vlxw;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v:
  ID = Intrinsic::riscv_th_vlxw;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v_tum:
  ID = Intrinsic::riscv_th_vlxw_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v_m:
  ID = Intrinsic::riscv_th_vlxw_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v_tumu:
  ID = Intrinsic::riscv_th_vlxw_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxw_v_mu:
  ID = Intrinsic::riscv_th_vlxw_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v_tu:
  ID = Intrinsic::riscv_th_vlxwu;
  PolicyAttrs = 2;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v:
  ID = Intrinsic::riscv_th_vlxwu;
  PolicyAttrs = 3;
  Ops[0] = Builder.CreateBitCast(Ops[0], ResultType->getPointerTo());
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v_tum:
  ID = Intrinsic::riscv_th_vlxwu_mask;
  PolicyAttrs = 2;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v_m:
  ID = Intrinsic::riscv_th_vlxwu_mask;
  PolicyAttrs = 3;
  Ops[1] = Builder.CreateBitCast(Ops[1], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v_tumu:
  ID = Intrinsic::riscv_th_vlxwu_mask;
  PolicyAttrs = 0;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vlxwu_v_mu:
  ID = Intrinsic::riscv_th_vlxwu_mask;
  PolicyAttrs = 1;
  Ops[2] = Builder.CreateBitCast(Ops[2], ResultType->getPointerTo());
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vmacc_vx:
  ID = Intrinsic::riscv_th_vmacc;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmacc_vx_m:
  ID = Intrinsic::riscv_th_vmacc_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmadc_vvm:
case RISCVVector::BI__builtin_rvv_th_vmadc_vxm:
  ID = Intrinsic::riscv_th_vmadc_carry_in;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmadd_vv:
case RISCVVector::BI__builtin_rvv_th_vmadd_vx:
  ID = Intrinsic::riscv_th_vmadd;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmadd_vx_m:
  ID = Intrinsic::riscv_th_vmadd_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmmv_m:
  ID = Intrinsic::riscv_th_vmand;
  PolicyAttrs = 3;
IsMasked = false;

      {
        // op1, vl
        IntrinsicTypes = {ResultType,
                          Ops[1]->getType()};
        Ops.insert(Ops.begin() + 1, Ops[0]);
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vmand_mm:
  ID = Intrinsic::riscv_th_vmand;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmandnot_mm:
  ID = Intrinsic::riscv_th_vmandnot;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmax_vx_tu:
  ID = Intrinsic::riscv_th_vmax;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv:
case RISCVVector::BI__builtin_rvv_th_vmax_vx:
  ID = Intrinsic::riscv_th_vmax;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmax_vx_tum:
  ID = Intrinsic::riscv_th_vmax_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmax_vx_m:
  ID = Intrinsic::riscv_th_vmax_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmax_vx_tumu:
  ID = Intrinsic::riscv_th_vmax_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmax_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmax_vx_mu:
  ID = Intrinsic::riscv_th_vmax_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx_tu:
  ID = Intrinsic::riscv_th_vmaxu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx:
  ID = Intrinsic::riscv_th_vmaxu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx_tum:
  ID = Intrinsic::riscv_th_vmaxu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx_m:
  ID = Intrinsic::riscv_th_vmaxu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx_tumu:
  ID = Intrinsic::riscv_th_vmaxu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmaxu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmaxu_vx_mu:
  ID = Intrinsic::riscv_th_vmaxu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmclr_m:
  ID = Intrinsic::riscv_th_vmclr;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmerge_vvm_tu:
case RISCVVector::BI__builtin_rvv_th_vmerge_vxm_tu:
  ID = Intrinsic::riscv_th_vmerge;
  PolicyAttrs = 2;
IsMasked = false;

    {
      // Builtin: (mask, op1, op2, vl), Intrinsic: (passthru, op1, op2, mask, vl)
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType)); // passthru
      IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vmerge_vvm:
case RISCVVector::BI__builtin_rvv_th_vmerge_vxm:
  ID = Intrinsic::riscv_th_vmerge;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Builtin: (mask, op1, op2, vl), Intrinsic: (passthru, op1, op2, mask, vl)
      if (PolicyAttrs & RVV_VTA)
        Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType)); // passthru
      IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
    }
    break;
case RISCVVector::BI__builtin_rvv_th_vmfeq_vv:
case RISCVVector::BI__builtin_rvv_th_vmfeq_vf:
  ID = Intrinsic::riscv_th_vmfeq;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfeq_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmfeq_vf_m:
  ID = Intrinsic::riscv_th_vmfeq_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfeq_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmfeq_vf_mu:
  ID = Intrinsic::riscv_th_vmfeq_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfge_vv:
case RISCVVector::BI__builtin_rvv_th_vmfge_vf:
  ID = Intrinsic::riscv_th_vmfge;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfge_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmfge_vf_m:
  ID = Intrinsic::riscv_th_vmfge_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfge_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmfge_vf_mu:
  ID = Intrinsic::riscv_th_vmfge_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfgt_vv:
case RISCVVector::BI__builtin_rvv_th_vmfgt_vf:
  ID = Intrinsic::riscv_th_vmfgt;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfgt_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmfgt_vf_m:
  ID = Intrinsic::riscv_th_vmfgt_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfgt_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmfgt_vf_mu:
  ID = Intrinsic::riscv_th_vmfgt_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfirst_m:
  ID = Intrinsic::riscv_th_vmfirst;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vfirst_m_m:
  ID = Intrinsic::riscv_th_vmfirst_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[0]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfle_vv:
case RISCVVector::BI__builtin_rvv_th_vmfle_vf:
  ID = Intrinsic::riscv_th_vmfle;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfle_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmfle_vf_m:
  ID = Intrinsic::riscv_th_vmfle_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfle_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmfle_vf_mu:
  ID = Intrinsic::riscv_th_vmfle_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmflt_vv:
case RISCVVector::BI__builtin_rvv_th_vmflt_vf:
  ID = Intrinsic::riscv_th_vmflt;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmflt_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmflt_vf_m:
  ID = Intrinsic::riscv_th_vmflt_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmflt_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmflt_vf_mu:
  ID = Intrinsic::riscv_th_vmflt_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfne_vv:
case RISCVVector::BI__builtin_rvv_th_vmfne_vf:
  ID = Intrinsic::riscv_th_vmfne;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfne_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmfne_vf_m:
  ID = Intrinsic::riscv_th_vmfne_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmfne_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmfne_vf_mu:
  ID = Intrinsic::riscv_th_vmfne_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmin_vx_tu:
  ID = Intrinsic::riscv_th_vmin;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv:
case RISCVVector::BI__builtin_rvv_th_vmin_vx:
  ID = Intrinsic::riscv_th_vmin;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmin_vx_tum:
  ID = Intrinsic::riscv_th_vmin_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmin_vx_m:
  ID = Intrinsic::riscv_th_vmin_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmin_vx_tumu:
  ID = Intrinsic::riscv_th_vmin_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmin_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmin_vx_mu:
  ID = Intrinsic::riscv_th_vmin_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vminu_vx_tu:
  ID = Intrinsic::riscv_th_vminu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv:
case RISCVVector::BI__builtin_rvv_th_vminu_vx:
  ID = Intrinsic::riscv_th_vminu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vminu_vx_tum:
  ID = Intrinsic::riscv_th_vminu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vminu_vx_m:
  ID = Intrinsic::riscv_th_vminu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vminu_vx_tumu:
  ID = Intrinsic::riscv_th_vminu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vminu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vminu_vx_mu:
  ID = Intrinsic::riscv_th_vminu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmnot_m:
  ID = Intrinsic::riscv_th_vmnand;
  PolicyAttrs = 3;
IsMasked = false;

      {
        // op1, vl
        IntrinsicTypes = {ResultType,
                          Ops[1]->getType()};
        Ops.insert(Ops.begin() + 1, Ops[0]);
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vmnand_mm:
  ID = Intrinsic::riscv_th_vmnand;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmnor_mm:
  ID = Intrinsic::riscv_th_vmnor;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmor_mm:
  ID = Intrinsic::riscv_th_vmor;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmornot_mm:
  ID = Intrinsic::riscv_th_vmornot;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vcpop_m:
  ID = Intrinsic::riscv_th_vmpopc;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vcpop_m_m:
  ID = Intrinsic::riscv_th_vmpopc_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[0]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsbc_vvm:
case RISCVVector::BI__builtin_rvv_th_vmsbc_vxm:
  ID = Intrinsic::riscv_th_vmsbc_borrow_in;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsbf_m:
  ID = Intrinsic::riscv_th_vmsbf;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsbf_m_m:
  ID = Intrinsic::riscv_th_vmsbf_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsbf_m_mu:
  ID = Intrinsic::riscv_th_vmsbf_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmseq_vv:
case RISCVVector::BI__builtin_rvv_th_vmseq_vx:
  ID = Intrinsic::riscv_th_vmseq;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmseq_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmseq_vx_m:
  ID = Intrinsic::riscv_th_vmseq_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmseq_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmseq_vx_mu:
  ID = Intrinsic::riscv_th_vmseq_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmset_m:
  ID = Intrinsic::riscv_th_vmset;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsge_vv:
case RISCVVector::BI__builtin_rvv_th_vmsge_vx:
  ID = Intrinsic::riscv_th_vmsge;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsge_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsge_vx_m:
  ID = Intrinsic::riscv_th_vmsge_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsge_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsge_vx_mu:
  ID = Intrinsic::riscv_th_vmsge_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vv:
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vx:
  ID = Intrinsic::riscv_th_vmsgeu;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vx_m:
  ID = Intrinsic::riscv_th_vmsgeu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsgeu_vx_mu:
  ID = Intrinsic::riscv_th_vmsgeu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgt_vv:
case RISCVVector::BI__builtin_rvv_th_vmsgt_vx:
  ID = Intrinsic::riscv_th_vmsgt;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgt_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsgt_vx_m:
  ID = Intrinsic::riscv_th_vmsgt_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgt_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsgt_vx_mu:
  ID = Intrinsic::riscv_th_vmsgt_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vv:
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vx:
  ID = Intrinsic::riscv_th_vmsgtu;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vx_m:
  ID = Intrinsic::riscv_th_vmsgtu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsgtu_vx_mu:
  ID = Intrinsic::riscv_th_vmsgtu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsif_m:
  ID = Intrinsic::riscv_th_vmsif;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsif_m_m:
  ID = Intrinsic::riscv_th_vmsif_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsif_m_mu:
  ID = Intrinsic::riscv_th_vmsif_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsle_vv:
case RISCVVector::BI__builtin_rvv_th_vmsle_vx:
  ID = Intrinsic::riscv_th_vmsle;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsle_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsle_vx_m:
  ID = Intrinsic::riscv_th_vmsle_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsle_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsle_vx_mu:
  ID = Intrinsic::riscv_th_vmsle_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsleu_vv:
case RISCVVector::BI__builtin_rvv_th_vmsleu_vx:
  ID = Intrinsic::riscv_th_vmsleu;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsleu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsleu_vx_m:
  ID = Intrinsic::riscv_th_vmsleu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsleu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsleu_vx_mu:
  ID = Intrinsic::riscv_th_vmsleu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmslt_vv:
case RISCVVector::BI__builtin_rvv_th_vmslt_vx:
  ID = Intrinsic::riscv_th_vmslt;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmslt_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmslt_vx_m:
  ID = Intrinsic::riscv_th_vmslt_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmslt_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmslt_vx_mu:
  ID = Intrinsic::riscv_th_vmslt_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsltu_vv:
case RISCVVector::BI__builtin_rvv_th_vmsltu_vx:
  ID = Intrinsic::riscv_th_vmsltu;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsltu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsltu_vx_m:
  ID = Intrinsic::riscv_th_vmsltu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsltu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsltu_vx_mu:
  ID = Intrinsic::riscv_th_vmsltu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsne_vv:
case RISCVVector::BI__builtin_rvv_th_vmsne_vx:
  ID = Intrinsic::riscv_th_vmsne;
  PolicyAttrs = 3;
  IntrinsicTypes = {Ops[0]->getType(), Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsne_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmsne_vx_m:
  ID = Intrinsic::riscv_th_vmsne_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsne_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmsne_vx_mu:
  ID = Intrinsic::riscv_th_vmsne_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsof_m:
  ID = Intrinsic::riscv_th_vmsof;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsof_m_m:
  ID = Intrinsic::riscv_th_vmsof_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmsof_m_mu:
  ID = Intrinsic::riscv_th_vmsof_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmul_vx_tu:
  ID = Intrinsic::riscv_th_vmul;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv:
case RISCVVector::BI__builtin_rvv_th_vmul_vx:
  ID = Intrinsic::riscv_th_vmul;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmul_vx_tum:
  ID = Intrinsic::riscv_th_vmul_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmul_vx_m:
  ID = Intrinsic::riscv_th_vmul_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmul_vx_tumu:
  ID = Intrinsic::riscv_th_vmul_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmul_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmul_vx_mu:
  ID = Intrinsic::riscv_th_vmul_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx_tu:
  ID = Intrinsic::riscv_th_vmulh;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx:
  ID = Intrinsic::riscv_th_vmulh;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx_tum:
  ID = Intrinsic::riscv_th_vmulh_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx_m:
  ID = Intrinsic::riscv_th_vmulh_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx_tumu:
  ID = Intrinsic::riscv_th_vmulh_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulh_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmulh_vx_mu:
  ID = Intrinsic::riscv_th_vmulh_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx_tu:
  ID = Intrinsic::riscv_th_vmulhsu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx:
  ID = Intrinsic::riscv_th_vmulhsu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx_tum:
  ID = Intrinsic::riscv_th_vmulhsu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx_m:
  ID = Intrinsic::riscv_th_vmulhsu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx_tumu:
  ID = Intrinsic::riscv_th_vmulhsu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmulhsu_vx_mu:
  ID = Intrinsic::riscv_th_vmulhsu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx_tu:
  ID = Intrinsic::riscv_th_vmulhu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx:
  ID = Intrinsic::riscv_th_vmulhu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx_tum:
  ID = Intrinsic::riscv_th_vmulhu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx_m:
  ID = Intrinsic::riscv_th_vmulhu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx_tumu:
  ID = Intrinsic::riscv_th_vmulhu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmulhu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vmulhu_vx_mu:
  ID = Intrinsic::riscv_th_vmulhu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmv_v_v_tu:
  ID = Intrinsic::riscv_th_vmv_v_v;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmv_v_v:
  ID = Intrinsic::riscv_th_vmv_v_v;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmv_v_x_tu:
  ID = Intrinsic::riscv_th_vmv_v_x;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmv_v_x:
  ID = Intrinsic::riscv_th_vmv_v_x;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmxnor_mm:
  ID = Intrinsic::riscv_th_vmxnor;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vmxor_mm:
  ID = Intrinsic::riscv_th_vmxor;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx_tu:
  ID = Intrinsic::riscv_th_vnclip;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx:
  ID = Intrinsic::riscv_th_vnclip;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx_tum:
  ID = Intrinsic::riscv_th_vnclip_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv_m:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx_m:
  ID = Intrinsic::riscv_th_vnclip_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx_tumu:
  ID = Intrinsic::riscv_th_vnclip_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclip_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vnclip_wx_mu:
  ID = Intrinsic::riscv_th_vnclip_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx_tu:
  ID = Intrinsic::riscv_th_vnclipu;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx:
  ID = Intrinsic::riscv_th_vnclipu;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx_tum:
  ID = Intrinsic::riscv_th_vnclipu_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv_m:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx_m:
  ID = Intrinsic::riscv_th_vnclipu_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx_tumu:
  ID = Intrinsic::riscv_th_vnclipu_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnclipu_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vnclipu_wx_mu:
  ID = Intrinsic::riscv_th_vnclipu_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, vxrm, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl)

    // Clang builtin
    // Unmask:   (op1, op2, vxrm, vl)
    // Mask:     (mask, op1, op2, vxrm, vl)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    // TODO: no policy in LLVM side for masked intrinsics.
    // if (IsMasked)
    //  Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));

    IntrinsicTypes = {ResultType, Ops[Offset]->getType(), Ops[Offset + 1]->getType(),
                      Ops.back()->getType()};
    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vnmsac_vv:
case RISCVVector::BI__builtin_rvv_th_vnmsac_vx:
  ID = Intrinsic::riscv_th_vnmsac;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnmsac_vv_m:
case RISCVVector::BI__builtin_rvv_th_vnmsac_vx_m:
  ID = Intrinsic::riscv_th_vnmsac_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnmsub_vv:
case RISCVVector::BI__builtin_rvv_th_vnmsub_vx:
  ID = Intrinsic::riscv_th_vnmsub;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnmsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vnmsub_vx_m:
  ID = Intrinsic::riscv_th_vnmsub_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx_tu:
  ID = Intrinsic::riscv_th_vnsra;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx:
  ID = Intrinsic::riscv_th_vnsra;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx_tum:
  ID = Intrinsic::riscv_th_vnsra_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv_m:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx_m:
  ID = Intrinsic::riscv_th_vnsra_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx_tumu:
  ID = Intrinsic::riscv_th_vnsra_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsra_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vnsra_wx_mu:
  ID = Intrinsic::riscv_th_vnsra_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx_tu:
  ID = Intrinsic::riscv_th_vnsrl;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx:
  ID = Intrinsic::riscv_th_vnsrl;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx_tum:
  ID = Intrinsic::riscv_th_vnsrl_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv_m:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx_m:
  ID = Intrinsic::riscv_th_vnsrl_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx_tumu:
  ID = Intrinsic::riscv_th_vnsrl_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnsrl_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vnsrl_wx_mu:
  ID = Intrinsic::riscv_th_vnsrl_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vor_vx_tu:
  ID = Intrinsic::riscv_th_vor;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv:
case RISCVVector::BI__builtin_rvv_th_vor_vx:
  ID = Intrinsic::riscv_th_vor;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vor_vx_tum:
  ID = Intrinsic::riscv_th_vor_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv_m:
case RISCVVector::BI__builtin_rvv_th_vor_vx_m:
  ID = Intrinsic::riscv_th_vor_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vor_vx_tumu:
  ID = Intrinsic::riscv_th_vor_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vor_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vor_vx_mu:
  ID = Intrinsic::riscv_th_vor_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredand_vs_tu:
  ID = Intrinsic::riscv_th_vredand;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredand_vs:
  ID = Intrinsic::riscv_th_vredand;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredand_vs_tum:
  ID = Intrinsic::riscv_th_vredand_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredand_vs_m:
  ID = Intrinsic::riscv_th_vredand_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmax_vs_tu:
  ID = Intrinsic::riscv_th_vredmax;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmax_vs:
  ID = Intrinsic::riscv_th_vredmax;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmax_vs_tum:
  ID = Intrinsic::riscv_th_vredmax_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmax_vs_m:
  ID = Intrinsic::riscv_th_vredmax_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmaxu_vs_tu:
  ID = Intrinsic::riscv_th_vredmaxu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmaxu_vs:
  ID = Intrinsic::riscv_th_vredmaxu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmaxu_vs_tum:
  ID = Intrinsic::riscv_th_vredmaxu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmaxu_vs_m:
  ID = Intrinsic::riscv_th_vredmaxu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmin_vs_tu:
  ID = Intrinsic::riscv_th_vredmin;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmin_vs:
  ID = Intrinsic::riscv_th_vredmin;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmin_vs_tum:
  ID = Intrinsic::riscv_th_vredmin_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredmin_vs_m:
  ID = Intrinsic::riscv_th_vredmin_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredminu_vs_tu:
  ID = Intrinsic::riscv_th_vredminu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredminu_vs:
  ID = Intrinsic::riscv_th_vredminu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredminu_vs_tum:
  ID = Intrinsic::riscv_th_vredminu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredminu_vs_m:
  ID = Intrinsic::riscv_th_vredminu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredor_vs_tu:
  ID = Intrinsic::riscv_th_vredor;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredor_vs:
  ID = Intrinsic::riscv_th_vredor;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredor_vs_tum:
  ID = Intrinsic::riscv_th_vredor_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredor_vs_m:
  ID = Intrinsic::riscv_th_vredor_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredsum_vs_tu:
  ID = Intrinsic::riscv_th_vredsum;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredsum_vs:
  ID = Intrinsic::riscv_th_vredsum;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredsum_vs_tum:
  ID = Intrinsic::riscv_th_vredsum_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredsum_vs_m:
  ID = Intrinsic::riscv_th_vredsum_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredxor_vs_tu:
  ID = Intrinsic::riscv_th_vredxor;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredxor_vs:
  ID = Intrinsic::riscv_th_vredxor;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredxor_vs_tum:
  ID = Intrinsic::riscv_th_vredxor_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vredxor_vs_m:
  ID = Intrinsic::riscv_th_vredxor_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vrem_vx_tu:
  ID = Intrinsic::riscv_th_vrem;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv:
case RISCVVector::BI__builtin_rvv_th_vrem_vx:
  ID = Intrinsic::riscv_th_vrem;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vrem_vx_tum:
  ID = Intrinsic::riscv_th_vrem_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv_m:
case RISCVVector::BI__builtin_rvv_th_vrem_vx_m:
  ID = Intrinsic::riscv_th_vrem_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vrem_vx_tumu:
  ID = Intrinsic::riscv_th_vrem_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrem_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vrem_vx_mu:
  ID = Intrinsic::riscv_th_vrem_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vremu_vx_tu:
  ID = Intrinsic::riscv_th_vremu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv:
case RISCVVector::BI__builtin_rvv_th_vremu_vx:
  ID = Intrinsic::riscv_th_vremu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vremu_vx_tum:
  ID = Intrinsic::riscv_th_vremu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vremu_vx_m:
  ID = Intrinsic::riscv_th_vremu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vremu_vx_tumu:
  ID = Intrinsic::riscv_th_vremu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vremu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vremu_vx_mu:
  ID = Intrinsic::riscv_th_vremu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv_tu:
  ID = Intrinsic::riscv_th_vrgather_vv;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv:
  ID = Intrinsic::riscv_th_vrgather_vv;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv_tum:
  ID = Intrinsic::riscv_th_vrgather_vv_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv_m:
  ID = Intrinsic::riscv_th_vrgather_vv_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv_tumu:
  ID = Intrinsic::riscv_th_vrgather_vv_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vv_mu:
  ID = Intrinsic::riscv_th_vrgather_vv_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx_tu:
  ID = Intrinsic::riscv_th_vrgather_vx;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx:
  ID = Intrinsic::riscv_th_vrgather_vx;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx_tum:
  ID = Intrinsic::riscv_th_vrgather_vx_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx_m:
  ID = Intrinsic::riscv_th_vrgather_vx_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx_tumu:
  ID = Intrinsic::riscv_th_vrgather_vx_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vrgather_vx_mu:
  ID = Intrinsic::riscv_th_vrgather_vx_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v_tu:
  ID = Intrinsic::riscv_th_vrsub;
  PolicyAttrs = 2;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx_tu:
  ID = Intrinsic::riscv_th_vrsub;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v:
  ID = Intrinsic::riscv_th_vrsub;
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx:
  ID = Intrinsic::riscv_th_vrsub;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v_tum:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 2;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx_tum:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v_m:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 3;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx_m:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v_tumu:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 0;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx_tumu:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vneg_v_mu:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 1;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2, llvm::Constant::getNullValue(ElemTy));

        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, op2, mask, vl, policy
          IntrinsicTypes = {ResultType, ElemTy, Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType, ElemTy, Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vrsub_vx_mu:
  ID = Intrinsic::riscv_th_vrsub_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx_tu:
  ID = Intrinsic::riscv_th_vsadd;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx:
  ID = Intrinsic::riscv_th_vsadd;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx_tum:
  ID = Intrinsic::riscv_th_vsadd_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx_m:
  ID = Intrinsic::riscv_th_vsadd_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx_tumu:
  ID = Intrinsic::riscv_th_vsadd_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsadd_vx_mu:
  ID = Intrinsic::riscv_th_vsadd_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx_tu:
  ID = Intrinsic::riscv_th_vsaddu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx:
  ID = Intrinsic::riscv_th_vsaddu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx_tum:
  ID = Intrinsic::riscv_th_vsaddu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx_m:
  ID = Intrinsic::riscv_th_vsaddu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx_tumu:
  ID = Intrinsic::riscv_th_vsaddu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsaddu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsaddu_vx_mu:
  ID = Intrinsic::riscv_th_vsaddu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsb_v:
  ID = Intrinsic::riscv_th_vsb;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsb_v_m:
  ID = Intrinsic::riscv_th_vsb_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsbc_vvm_tu:
case RISCVVector::BI__builtin_rvv_th_vsbc_vxm_tu:
  ID = Intrinsic::riscv_th_vsbc;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsbc_vvm:
case RISCVVector::BI__builtin_rvv_th_vsbc_vxm:
  ID = Intrinsic::riscv_th_vsbc;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vse8_v:
case RISCVVector::BI__builtin_rvv_th_vse16_v:
case RISCVVector::BI__builtin_rvv_th_vse32_v:
case RISCVVector::BI__builtin_rvv_th_vse64_v:
  ID = Intrinsic::riscv_th_vse;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vse8_v_m:
case RISCVVector::BI__builtin_rvv_th_vse16_v_m:
case RISCVVector::BI__builtin_rvv_th_vse32_v_m:
case RISCVVector::BI__builtin_rvv_th_vse64_v_m:
  ID = Intrinsic::riscv_th_vse_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsetvl:
  ID = Intrinsic::riscv_th_vsetvl;
  PolicyAttrs = 3;
IsMasked = false;
IntrinsicTypes = {ResultType};break;
case RISCVVector::BI__builtin_rvv_th_vsetvlmax:
  ID = Intrinsic::riscv_th_vsetvlmax;
  PolicyAttrs = 3;
IsMasked = false;
IntrinsicTypes = {ResultType};break;
case RISCVVector::BI__builtin_rvv_th_vsh_v:
  ID = Intrinsic::riscv_th_vsh;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsh_v_m:
  ID = Intrinsic::riscv_th_vsh_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx_tu:
  ID = Intrinsic::riscv_th_vslide1down;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx:
  ID = Intrinsic::riscv_th_vslide1down;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx_tum:
  ID = Intrinsic::riscv_th_vslide1down_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx_m:
  ID = Intrinsic::riscv_th_vslide1down_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx_tumu:
  ID = Intrinsic::riscv_th_vslide1down_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1down_vx_mu:
  ID = Intrinsic::riscv_th_vslide1down_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx_tu:
  ID = Intrinsic::riscv_th_vslide1up;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx:
  ID = Intrinsic::riscv_th_vslide1up;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx_tum:
  ID = Intrinsic::riscv_th_vslide1up_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx_m:
  ID = Intrinsic::riscv_th_vslide1up_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx_tumu:
  ID = Intrinsic::riscv_th_vslide1up_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslide1up_vx_mu:
  ID = Intrinsic::riscv_th_vslide1up_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vslidedown_vx:
  ID = Intrinsic::riscv_th_vslidedown;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
        if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      } else {
        if (PolicyAttrs & RVV_VTA)
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslidedown_vx_tum:
  ID = Intrinsic::riscv_th_vslidedown_mask;
  PolicyAttrs = 2;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
        if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      } else {
        if (PolicyAttrs & RVV_VTA)
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslidedown_vx_m:
  ID = Intrinsic::riscv_th_vslidedown_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
        if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      } else {
        if (PolicyAttrs & RVV_VTA)
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslidedown_vx_tumu:
  ID = Intrinsic::riscv_th_vslidedown_mask;
  PolicyAttrs = 0;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
        if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      } else {
        if (PolicyAttrs & RVV_VTA)
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslidedown_vx_mu:
  ID = Intrinsic::riscv_th_vslidedown_mask;
  PolicyAttrs = 1;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
        if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      } else {
        if (PolicyAttrs & RVV_VTA)
          Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslideup_vx:
  ID = Intrinsic::riscv_th_vslideup;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslideup_vx_tum:
  ID = Intrinsic::riscv_th_vslideup_mask;
  PolicyAttrs = 2;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslideup_vx_m:
  ID = Intrinsic::riscv_th_vslideup_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslideup_vx_tumu:
  ID = Intrinsic::riscv_th_vslideup_mask;
  PolicyAttrs = 0;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vslideup_vx_mu:
  ID = Intrinsic::riscv_th_vslideup_mask;
  PolicyAttrs = 1;
IsMasked = true;

      if (IsMasked) {
        std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
      }

      // TODO: no policy in LLVM side for masked intrinsics.
      // Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops.back()->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsll_vx_tu:
  ID = Intrinsic::riscv_th_vsll;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv:
case RISCVVector::BI__builtin_rvv_th_vsll_vx:
  ID = Intrinsic::riscv_th_vsll;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsll_vx_tum:
  ID = Intrinsic::riscv_th_vsll_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsll_vx_m:
  ID = Intrinsic::riscv_th_vsll_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsll_vx_tumu:
  ID = Intrinsic::riscv_th_vsll_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsll_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsll_vx_mu:
  ID = Intrinsic::riscv_th_vsll_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx_tu:
  ID = Intrinsic::riscv_th_vsmul;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx:
  ID = Intrinsic::riscv_th_vsmul;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx_tum:
  ID = Intrinsic::riscv_th_vsmul_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx_m:
  ID = Intrinsic::riscv_th_vsmul_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx_tumu:
  ID = Intrinsic::riscv_th_vsmul_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsmul_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsmul_vx_mu:
  ID = Intrinsic::riscv_th_vsmul_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsra_vx_tu:
  ID = Intrinsic::riscv_th_vsra;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv:
case RISCVVector::BI__builtin_rvv_th_vsra_vx:
  ID = Intrinsic::riscv_th_vsra;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsra_vx_tum:
  ID = Intrinsic::riscv_th_vsra_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsra_vx_m:
  ID = Intrinsic::riscv_th_vsra_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsra_vx_tumu:
  ID = Intrinsic::riscv_th_vsra_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsra_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsra_vx_mu:
  ID = Intrinsic::riscv_th_vsra_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx_tu:
  ID = Intrinsic::riscv_th_vsrl;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx:
  ID = Intrinsic::riscv_th_vsrl;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx_tum:
  ID = Intrinsic::riscv_th_vsrl_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx_m:
  ID = Intrinsic::riscv_th_vsrl_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx_tumu:
  ID = Intrinsic::riscv_th_vsrl_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsrl_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsrl_vx_mu:
  ID = Intrinsic::riscv_th_vsrl_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssb_v:
  ID = Intrinsic::riscv_th_vssb;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vssb_v_m:
  ID = Intrinsic::riscv_th_vssb_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsse8_v:
case RISCVVector::BI__builtin_rvv_th_vsse16_v:
case RISCVVector::BI__builtin_rvv_th_vsse32_v:
case RISCVVector::BI__builtin_rvv_th_vsse64_v:
  ID = Intrinsic::riscv_th_vsse;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsse8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsse16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsse32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsse64_v_m:
  ID = Intrinsic::riscv_th_vsse_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsseg2b_v:
  ID = Intrinsic::riscv_th_vsseg2b;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2b_v_m:
  ID = Intrinsic::riscv_th_vsseg2b_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg2e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg2e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg2e64_v:
  ID = Intrinsic::riscv_th_vsseg2e;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg2e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg2e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg2e64_v_m:
  ID = Intrinsic::riscv_th_vsseg2e_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2h_v:
  ID = Intrinsic::riscv_th_vsseg2h;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2h_v_m:
  ID = Intrinsic::riscv_th_vsseg2h_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2w_v:
  ID = Intrinsic::riscv_th_vsseg2w;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg2w_v_m:
  ID = Intrinsic::riscv_th_vsseg2w_mask;
  NF = 2;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3b_v:
  ID = Intrinsic::riscv_th_vsseg3b;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3b_v_m:
  ID = Intrinsic::riscv_th_vsseg3b_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg3e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg3e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg3e64_v:
  ID = Intrinsic::riscv_th_vsseg3e;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg3e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg3e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg3e64_v_m:
  ID = Intrinsic::riscv_th_vsseg3e_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3h_v:
  ID = Intrinsic::riscv_th_vsseg3h;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3h_v_m:
  ID = Intrinsic::riscv_th_vsseg3h_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3w_v:
  ID = Intrinsic::riscv_th_vsseg3w;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg3w_v_m:
  ID = Intrinsic::riscv_th_vsseg3w_mask;
  NF = 3;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4b_v:
  ID = Intrinsic::riscv_th_vsseg4b;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4b_v_m:
  ID = Intrinsic::riscv_th_vsseg4b_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg4e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg4e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg4e64_v:
  ID = Intrinsic::riscv_th_vsseg4e;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg4e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg4e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg4e64_v_m:
  ID = Intrinsic::riscv_th_vsseg4e_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4h_v:
  ID = Intrinsic::riscv_th_vsseg4h;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4h_v_m:
  ID = Intrinsic::riscv_th_vsseg4h_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4w_v:
  ID = Intrinsic::riscv_th_vsseg4w;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg4w_v_m:
  ID = Intrinsic::riscv_th_vsseg4w_mask;
  NF = 4;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5b_v:
  ID = Intrinsic::riscv_th_vsseg5b;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5b_v_m:
  ID = Intrinsic::riscv_th_vsseg5b_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg5e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg5e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg5e64_v:
  ID = Intrinsic::riscv_th_vsseg5e;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg5e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg5e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg5e64_v_m:
  ID = Intrinsic::riscv_th_vsseg5e_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5h_v:
  ID = Intrinsic::riscv_th_vsseg5h;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5h_v_m:
  ID = Intrinsic::riscv_th_vsseg5h_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5w_v:
  ID = Intrinsic::riscv_th_vsseg5w;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg5w_v_m:
  ID = Intrinsic::riscv_th_vsseg5w_mask;
  NF = 5;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6b_v:
  ID = Intrinsic::riscv_th_vsseg6b;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6b_v_m:
  ID = Intrinsic::riscv_th_vsseg6b_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg6e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg6e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg6e64_v:
  ID = Intrinsic::riscv_th_vsseg6e;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg6e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg6e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg6e64_v_m:
  ID = Intrinsic::riscv_th_vsseg6e_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6h_v:
  ID = Intrinsic::riscv_th_vsseg6h;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6h_v_m:
  ID = Intrinsic::riscv_th_vsseg6h_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6w_v:
  ID = Intrinsic::riscv_th_vsseg6w;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg6w_v_m:
  ID = Intrinsic::riscv_th_vsseg6w_mask;
  NF = 6;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7b_v:
  ID = Intrinsic::riscv_th_vsseg7b;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7b_v_m:
  ID = Intrinsic::riscv_th_vsseg7b_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg7e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg7e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg7e64_v:
  ID = Intrinsic::riscv_th_vsseg7e;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg7e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg7e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg7e64_v_m:
  ID = Intrinsic::riscv_th_vsseg7e_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7h_v:
  ID = Intrinsic::riscv_th_vsseg7h;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7h_v_m:
  ID = Intrinsic::riscv_th_vsseg7h_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7w_v:
  ID = Intrinsic::riscv_th_vsseg7w;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg7w_v_m:
  ID = Intrinsic::riscv_th_vsseg7w_mask;
  NF = 7;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8b_v:
  ID = Intrinsic::riscv_th_vsseg8b;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8b_v_m:
  ID = Intrinsic::riscv_th_vsseg8b_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8e8_v:
case RISCVVector::BI__builtin_rvv_th_vsseg8e16_v:
case RISCVVector::BI__builtin_rvv_th_vsseg8e32_v:
case RISCVVector::BI__builtin_rvv_th_vsseg8e64_v:
  ID = Intrinsic::riscv_th_vsseg8e;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8e8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg8e16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg8e32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsseg8e64_v_m:
  ID = Intrinsic::riscv_th_vsseg8e_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8h_v:
  ID = Intrinsic::riscv_th_vsseg8h;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8h_v_m:
  ID = Intrinsic::riscv_th_vsseg8h_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8w_v:
  ID = Intrinsic::riscv_th_vsseg8w;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = false;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vsseg8w_v_m:
  ID = Intrinsic::riscv_th_vsseg8w_mask;
  NF = 8;
  PolicyAttrs = 3;
IsMasked = true;

    {
      // Masked
      // Builtin: (mask, ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, mask, vl)
      // Unmasked
      // Builtin: (ptr, v_tuple, vl)
      // Intrinsic: (val0, val1, ..., ptr, vl)
      unsigned Offset = IsMasked ? 1 : 0;
      llvm::Value *VTupleOperand = Ops[Offset + 1];

      SmallVector<llvm::Value*, 12> Operands;
      for (unsigned I = 0; I < NF; ++I) {
        llvm::Value *V = Builder.CreateExtractValue(VTupleOperand, {I});
        Operands.push_back(V);
      }
      Operands.push_back(Ops[Offset]); // Ptr
      if (IsMasked)
        Operands.push_back(Ops[0]);
      Operands.push_back(Ops[Offset + 2]); // VL

      IntrinsicTypes = {Operands[0]->getType(), Operands.back()->getType()};
      llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
      return Builder.CreateCall(F, Operands, "");
   }
      break;
case RISCVVector::BI__builtin_rvv_th_vssh_v:
  ID = Intrinsic::riscv_th_vssh;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vssh_v_m:
  ID = Intrinsic::riscv_th_vssh_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vssra_vx_tu:
  ID = Intrinsic::riscv_th_vssra;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv:
case RISCVVector::BI__builtin_rvv_th_vssra_vx:
  ID = Intrinsic::riscv_th_vssra;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vssra_vx_tum:
  ID = Intrinsic::riscv_th_vssra_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv_m:
case RISCVVector::BI__builtin_rvv_th_vssra_vx_m:
  ID = Intrinsic::riscv_th_vssra_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vssra_vx_tumu:
  ID = Intrinsic::riscv_th_vssra_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssra_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vssra_vx_mu:
  ID = Intrinsic::riscv_th_vssra_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx_tu:
  ID = Intrinsic::riscv_th_vssrl;
  PolicyAttrs = 2;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx:
  ID = Intrinsic::riscv_th_vssrl;
  PolicyAttrs = 3;
IsMasked = false;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx_tum:
  ID = Intrinsic::riscv_th_vssrl_mask;
  PolicyAttrs = 2;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv_m:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx_m:
  ID = Intrinsic::riscv_th_vssrl_mask;
  PolicyAttrs = 3;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx_tumu:
  ID = Intrinsic::riscv_th_vssrl_mask;
  PolicyAttrs = 0;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssrl_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vssrl_vx_mu:
  ID = Intrinsic::riscv_th_vssrl_mask;
  PolicyAttrs = 1;
IsMasked = true;

  {
    // LLVM intrinsic
    // Unmasked: (passthru, op0, op1, round_mode, vl)
    // Masked:   (passthru, vector_in, vector_in/scalar_in, mask, vxrm, vl, policy)

    SmallVector<llvm::Value*, 7> Operands;
    bool HasMaskedOff = !(
        (IsMasked && (PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA)) ||
        (!IsMasked && PolicyAttrs & RVV_VTA));
    unsigned Offset = IsMasked ?
        (HasMaskedOff ? 2 : 1) : (HasMaskedOff ? 1 : 0);

    if (!HasMaskedOff)
      Operands.push_back(llvm::PoisonValue::get(ResultType));
    else
      Operands.push_back(Ops[IsMasked ? 1 : 0]);

    Operands.push_back(Ops[Offset]); // op0
    Operands.push_back(Ops[Offset + 1]); // op1

    if (IsMasked)
      Operands.push_back(Ops[0]); // mask

    Operands.push_back(Ops[Offset + 2]); // vxrm
    Operands.push_back(Ops[Offset + 3]); // vl

    if (IsMasked) {
      // TODO: no policy in LLVM side for masked intrinsics.
      // Operands.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops[Offset + 3]->getType()};
    } else {
      IntrinsicTypes = {ResultType, Ops[Offset + 1]->getType(), Ops.back()->getType()};
    }

    llvm::Function *F = CGM.getIntrinsic(ID, IntrinsicTypes);
    return Builder.CreateCall(F, Operands, "");
  }
break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vssub_vx_tu:
  ID = Intrinsic::riscv_th_vssub;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv:
case RISCVVector::BI__builtin_rvv_th_vssub_vx:
  ID = Intrinsic::riscv_th_vssub;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vssub_vx_tum:
  ID = Intrinsic::riscv_th_vssub_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vssub_vx_m:
  ID = Intrinsic::riscv_th_vssub_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vssub_vx_tumu:
  ID = Intrinsic::riscv_th_vssub_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vssub_vx_mu:
  ID = Intrinsic::riscv_th_vssub_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx_tu:
  ID = Intrinsic::riscv_th_vssubu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx:
  ID = Intrinsic::riscv_th_vssubu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx_tum:
  ID = Intrinsic::riscv_th_vssubu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx_m:
  ID = Intrinsic::riscv_th_vssubu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx_tumu:
  ID = Intrinsic::riscv_th_vssubu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssubu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vssubu_vx_mu:
  ID = Intrinsic::riscv_th_vssubu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vssw_v:
  ID = Intrinsic::riscv_th_vssw;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vssw_v_m:
  ID = Intrinsic::riscv_th_vssw_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, stride, value, vl). Intrinsic: (value, ptr, stride, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, stride, value, vl). Intrinsic: (value, ptr, stride, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vsub_vx_tu:
  ID = Intrinsic::riscv_th_vsub;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv:
case RISCVVector::BI__builtin_rvv_th_vsub_vx:
  ID = Intrinsic::riscv_th_vsub;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vsub_vx_tum:
  ID = Intrinsic::riscv_th_vsub_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vsub_vx_m:
  ID = Intrinsic::riscv_th_vsub_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vsub_vx_tumu:
  ID = Intrinsic::riscv_th_vsub_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vsub_vx_mu:
  ID = Intrinsic::riscv_th_vsub_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vsw_v:
  ID = Intrinsic::riscv_th_vsw;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsw_v_m:
  ID = Intrinsic::riscv_th_vsw_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, value, vl). Intrinsic: (value, ptr, mask, vl)
        std::swap(Ops[0], Ops[2]);
      } else {
        // Builtin: (ptr, value, vl). Intrinsic: (value, ptr, vl)
        std::swap(Ops[0], Ops[1]);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[3]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxb_v:
  ID = Intrinsic::riscv_th_vsxb;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxb_v_m:
  ID = Intrinsic::riscv_th_vsxb_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsoxei8_v:
case RISCVVector::BI__builtin_rvv_th_vsoxei16_v:
case RISCVVector::BI__builtin_rvv_th_vsoxei32_v:
case RISCVVector::BI__builtin_rvv_th_vsoxei64_v:
  ID = Intrinsic::riscv_th_vsxe;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsoxei8_v_m:
case RISCVVector::BI__builtin_rvv_th_vsoxei16_v_m:
case RISCVVector::BI__builtin_rvv_th_vsoxei32_v_m:
case RISCVVector::BI__builtin_rvv_th_vsoxei64_v_m:
  ID = Intrinsic::riscv_th_vsxe_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxh_v:
  ID = Intrinsic::riscv_th_vsxh;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxh_v_m:
  ID = Intrinsic::riscv_th_vsxh_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxw_v:
  ID = Intrinsic::riscv_th_vsxw;
  PolicyAttrs = 3;
IsMasked = false;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vsxw_v_m:
  ID = Intrinsic::riscv_th_vsxw_mask;
  PolicyAttrs = 3;
IsMasked = true;

      if (IsMasked) {
        // Builtin: (mask, ptr, index, value, vl). Intrinsic: (value, ptr, index, mask, vl)
        std::swap(Ops[0], Ops[3]);
      } else {
        // Builtin: (ptr, index, value, vl). Intrinsic: (value, ptr, index, vl)
        std::rotate(Ops.begin(), Ops.begin() + 2, Ops.begin() + 3);
      }
      Ops[1] = Builder.CreateBitCast(Ops[1], Ops[0]->getType()->getPointerTo());
      if (IsMasked)
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[4]->getType()};
      else
        IntrinsicTypes = {Ops[0]->getType(), Ops[2]->getType(), Ops[3]->getType()};
    break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx_tu:
  ID = Intrinsic::riscv_th_vwadd;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx:
  ID = Intrinsic::riscv_th_vwadd;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx_tum:
  ID = Intrinsic::riscv_th_vwadd_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx_m:
  ID = Intrinsic::riscv_th_vwadd_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx_tumu:
  ID = Intrinsic::riscv_th_vwadd_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwadd_vx_mu:
  ID = Intrinsic::riscv_th_vwadd_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx_tu:
  ID = Intrinsic::riscv_th_vwadd_w;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx:
  ID = Intrinsic::riscv_th_vwadd_w;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx_tum:
  ID = Intrinsic::riscv_th_vwadd_w_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv_m:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx_m:
  ID = Intrinsic::riscv_th_vwadd_w_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx_tumu:
  ID = Intrinsic::riscv_th_vwadd_w_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwadd_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vwadd_wx_mu:
  ID = Intrinsic::riscv_th_vwadd_w_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx_tu:
  ID = Intrinsic::riscv_th_vwaddu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx:
  ID = Intrinsic::riscv_th_vwaddu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx_tum:
  ID = Intrinsic::riscv_th_vwaddu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx_m:
  ID = Intrinsic::riscv_th_vwaddu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx_tumu:
  ID = Intrinsic::riscv_th_vwaddu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_vx_mu:
  ID = Intrinsic::riscv_th_vwaddu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx_tu:
  ID = Intrinsic::riscv_th_vwaddu_w;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx:
  ID = Intrinsic::riscv_th_vwaddu_w;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx_tum:
  ID = Intrinsic::riscv_th_vwaddu_w_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv_m:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx_m:
  ID = Intrinsic::riscv_th_vwaddu_w_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx_tumu:
  ID = Intrinsic::riscv_th_vwaddu_w_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwaddu_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vwaddu_wx_mu:
  ID = Intrinsic::riscv_th_vwaddu_w_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmacc_vv:
case RISCVVector::BI__builtin_rvv_th_vwmacc_vx:
  ID = Intrinsic::riscv_th_vwmacc;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmacc_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmacc_vx_m:
  ID = Intrinsic::riscv_th_vwmacc_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccsu_vv:
case RISCVVector::BI__builtin_rvv_th_vwmaccsu_vx:
  ID = Intrinsic::riscv_th_vwmaccsu;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccsu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmaccsu_vx_m:
  ID = Intrinsic::riscv_th_vwmaccsu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccu_vv:
case RISCVVector::BI__builtin_rvv_th_vwmaccu_vx:
  ID = Intrinsic::riscv_th_vwmaccu;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmaccu_vx_m:
  ID = Intrinsic::riscv_th_vwmaccu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccus_vx:
  ID = Intrinsic::riscv_th_vwmaccus;
  PolicyAttrs = 3;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmaccus_vx_m:
  ID = Intrinsic::riscv_th_vwmaccus_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx_tu:
  ID = Intrinsic::riscv_th_vwmul;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx:
  ID = Intrinsic::riscv_th_vwmul;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx_tum:
  ID = Intrinsic::riscv_th_vwmul_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx_m:
  ID = Intrinsic::riscv_th_vwmul_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx_tumu:
  ID = Intrinsic::riscv_th_vwmul_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmul_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwmul_vx_mu:
  ID = Intrinsic::riscv_th_vwmul_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx_tu:
  ID = Intrinsic::riscv_th_vwmulsu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx:
  ID = Intrinsic::riscv_th_vwmulsu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx_tum:
  ID = Intrinsic::riscv_th_vwmulsu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx_m:
  ID = Intrinsic::riscv_th_vwmulsu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx_tumu:
  ID = Intrinsic::riscv_th_vwmulsu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwmulsu_vx_mu:
  ID = Intrinsic::riscv_th_vwmulsu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx_tu:
  ID = Intrinsic::riscv_th_vwmulu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx:
  ID = Intrinsic::riscv_th_vwmulu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx_tum:
  ID = Intrinsic::riscv_th_vwmulu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx_m:
  ID = Intrinsic::riscv_th_vwmulu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx_tumu:
  ID = Intrinsic::riscv_th_vwmulu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwmulu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwmulu_vx_mu:
  ID = Intrinsic::riscv_th_vwmulu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsum_vs_tu:
  ID = Intrinsic::riscv_th_vwredsum;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsum_vs:
  ID = Intrinsic::riscv_th_vwredsum;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsum_vs_tum:
  ID = Intrinsic::riscv_th_vwredsum_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsum_vs_m:
  ID = Intrinsic::riscv_th_vwredsum_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsumu_vs_tu:
  ID = Intrinsic::riscv_th_vwredsumu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsumu_vs:
  ID = Intrinsic::riscv_th_vwredsumu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsumu_vs_tum:
  ID = Intrinsic::riscv_th_vwredsumu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwredsumu_vs_m:
  ID = Intrinsic::riscv_th_vwredsumu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx_tu:
  ID = Intrinsic::riscv_th_vwsub;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx:
  ID = Intrinsic::riscv_th_vwsub;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx_tum:
  ID = Intrinsic::riscv_th_vwsub_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx_m:
  ID = Intrinsic::riscv_th_vwsub_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx_tumu:
  ID = Intrinsic::riscv_th_vwsub_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwsub_vx_mu:
  ID = Intrinsic::riscv_th_vwsub_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx_tu:
  ID = Intrinsic::riscv_th_vwsub_w;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx:
  ID = Intrinsic::riscv_th_vwsub_w;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx_tum:
  ID = Intrinsic::riscv_th_vwsub_w_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv_m:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx_m:
  ID = Intrinsic::riscv_th_vwsub_w_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx_tumu:
  ID = Intrinsic::riscv_th_vwsub_w_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsub_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vwsub_wx_mu:
  ID = Intrinsic::riscv_th_vwsub_w_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx_tu:
  ID = Intrinsic::riscv_th_vwsubu;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx:
  ID = Intrinsic::riscv_th_vwsubu;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx_tum:
  ID = Intrinsic::riscv_th_vwsubu_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv_m:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx_m:
  ID = Intrinsic::riscv_th_vwsubu_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx_tumu:
  ID = Intrinsic::riscv_th_vwsubu_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_vx_mu:
  ID = Intrinsic::riscv_th_vwsubu_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[1]->getType(), Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv_tu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx_tu:
  ID = Intrinsic::riscv_th_vwsubu_w;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx:
  ID = Intrinsic::riscv_th_vwsubu_w;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv_tum:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx_tum:
  ID = Intrinsic::riscv_th_vwsubu_w_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv_m:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx_m:
  ID = Intrinsic::riscv_th_vwsubu_w_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv_tumu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx_tumu:
  ID = Intrinsic::riscv_th_vwsubu_w_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vwsubu_wv_mu:
case RISCVVector::BI__builtin_rvv_th_vwsubu_wx_mu:
  ID = Intrinsic::riscv_th_vwsubu_w_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v_tu:
  ID = Intrinsic::riscv_th_vxor;
  PolicyAttrs = 2;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv_tu:
case RISCVVector::BI__builtin_rvv_th_vxor_vx_tu:
  ID = Intrinsic::riscv_th_vxor;
  PolicyAttrs = 2;
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v:
  ID = Intrinsic::riscv_th_vxor;
  PolicyAttrs = 3;
IsMasked = false;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv:
case RISCVVector::BI__builtin_rvv_th_vxor_vx:
  ID = Intrinsic::riscv_th_vxor;
  PolicyAttrs = 3;
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v_tum:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 2;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv_tum:
case RISCVVector::BI__builtin_rvv_th_vxor_vx_tum:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 2;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v_m:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 3;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv_m:
case RISCVVector::BI__builtin_rvv_th_vxor_vx_m:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 3;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v_tumu:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 0;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv_tumu:
case RISCVVector::BI__builtin_rvv_th_vxor_vx_tumu:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 0;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;
case RISCVVector::BI__builtin_rvv_th_vnot_v_mu:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 1;
IsMasked = true;

      {
        if (IsMasked) {
          std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
          if ((PolicyAttrs & RVV_VTA) && (PolicyAttrs & RVV_VMA))
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        } else {
          if (PolicyAttrs & RVV_VTA)
            Ops.insert(Ops.begin(), llvm::PoisonValue::get(ResultType));
        }
        auto ElemTy = cast<llvm::VectorType>(ResultType)->getElementType();
        Ops.insert(Ops.begin() + 2,
                   llvm::Constant::getAllOnesValue(ElemTy));
        if (IsMasked) {
          // TODO: no policy in LLVM side for masked intrinsics.
          Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
          // maskedoff, op1, po2, mask, vl, policy
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[4]->getType()};
        } else {
          // passthru, op1, op2, vl
          IntrinsicTypes = {ResultType,
                            ElemTy,
                            Ops[3]->getType()};
        }
        break;
      }
      break;
case RISCVVector::BI__builtin_rvv_th_vxor_vv_mu:
case RISCVVector::BI__builtin_rvv_th_vxor_vx_mu:
  ID = Intrinsic::riscv_th_vxor_mask;
  PolicyAttrs = 1;
  std::rotate(Ops.begin(), Ops.begin() + 1, Ops.end() - 1);
  Ops.push_back(ConstantInt::get(Ops.back()->getType(), PolicyAttrs));
  IntrinsicTypes = {ResultType, Ops[2]->getType(), Ops.back()->getType()};
  break;

